/*
 * ConfigFile.java, part of the WipeFreeSpaceGUI package.
 *
 * Copyright (C) 2009-2012 Bogdan Drozdowski, bogdandr (at) op.pl
 * License: GNU General Public License, v3+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foudation:
 *		Free Software Foundation
 *		51 Franklin Street, Fifth Floor
 *		Boston, MA 02110-1301
 *		USA
 *
 */

package BogDroSoft.wfs;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.PrintWriter;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * A class for reading and writing WipeFreeSpaceGUI configuration files.
 * @author Bogdan Drozdowski
 */
public class ConfigFile
{
	private File cfgFile;
	// wipefreespace parameters:
	private boolean allzeros;
	private int superOffset = -1;
	private int blockSize = -1;
	private boolean force;
	private int iterations = -1;
	private boolean lastZero;
	private boolean nopart;
	private boolean nounrm;
	private boolean nowfs;
	private boolean ioctl;
	private Vector<String> fsList;
	private String wfsPath;
	private boolean selectMethod;
	private String methodName;
	// main window's parameters:
	private int x;
	private int y;
	private int width;
	private int height;
	private boolean isMax;
	private int fontSize;

	// patterns for matching:
	private static final Pattern allzeroPat = Pattern.compile
			("all_zero\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);			// NOI18N
	private static final Pattern superPat = Pattern.compile
			("superblock_offset\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);	// NOI18N
	private static final Pattern bsizePat = Pattern.compile
			("blocksize\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);			// NOI18N
	private static final Pattern forcePat = Pattern.compile
			("force\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);				// NOI18N
	private static final Pattern iterPat = Pattern.compile
			("iterations\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);		// NOI18N
	private static final Pattern lastzeroPat = Pattern.compile
			("last_zero\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);			// NOI18N
	private static final Pattern nopartPat = Pattern.compile
			("no_part\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);			// NOI18N
	private static final Pattern nounrmPat = Pattern.compile
			("no_undel\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);			// NOI18N
	private static final Pattern nowfsPat = Pattern.compile
			("no_free\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);			// NOI18N
	private static final Pattern ioctlPat = Pattern.compile
			("no_cache\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);			// NOI18N
	private static final Pattern fsListPat = Pattern.compile
			("filesystems\\s*=\\s*(.*)", Pattern.CASE_INSENSITIVE);			// NOI18N
	private static final Pattern xPat = Pattern.compile
			("x\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);					// NOI18N
	private static final Pattern yPat = Pattern.compile
			("y\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);					// NOI18N
	private static final Pattern widthPat = Pattern.compile
			("width\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);				// NOI18N
	private static final Pattern heightPat = Pattern.compile
			("height\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);			// NOI18N
	private static final Pattern isMaxPat = Pattern.compile
			("ismax\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);				// NOI18N
	private static final Pattern fontSizePat = Pattern.compile
			("font_size\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);				// NOI18N
	private static final Pattern wfsPathPat = Pattern.compile
			("wfs_path\\s*=\\s*(.*)", Pattern.CASE_INSENSITIVE);				// NOI18N
	private static final Pattern isMethodPat = Pattern.compile
			("is_method\\s*=\\s*(\\d+)", Pattern.CASE_INSENSITIVE);				// NOI18N
	private static final Pattern methodNamePat = Pattern.compile
			("method\\s*=\\s*(.*)", Pattern.CASE_INSENSITIVE);				// NOI18N

	private Matcher allzeroM;
	private Matcher superM;
	private Matcher bsizeM;
	private Matcher forceM;
	private Matcher iterM;
	private Matcher lastzeroM;
	private Matcher nopartM;
	private Matcher nounrmM;
	private Matcher nowfsM;
	private Matcher ioctlM;
	private Matcher fsListM;
	private Matcher xM;
	private Matcher yM;
	private Matcher widthM;
	private Matcher heightM;
	private Matcher isMaxM;
	private Matcher fontSizeM;
	private Matcher wfsPathM;
	private Matcher isMethodM;
	private Matcher methodNameM;

	/**
	 * Creates a new instance of ConfigFile.
	 * @param f The file to read/write parameters from/to.
	 */
	public ConfigFile (File f)
	{
		if ( f == null )
			throw new NullPointerException ("ConfigFile:f==null");	// NOI18N
		cfgFile = f;
	}

	/**
	 * Read the parameters from the given file and stores them inside private
	 * fields.
	 * @throws Exception on file error.
	 */
	public void read () throws Exception
	{
		allzeros = false;
		superOffset = -1;
		blockSize = -1;
		force = false;
		iterations = -1;
		lastZero = false;
		nopart = false;
		nounrm = false;
		nowfs = false;
		ioctl = false;
		fsList = new Vector<String> (10);
		x = 0;
		y = 0;
		width = 800;
		height = 600;
		isMax = false;
		fontSize = 12;
		// don't force a specific encoding
		BufferedReader br = new BufferedReader (new FileReader (cfgFile));
		String line;
		do
		{
			line = null;
			try
			{
				line = br.readLine ();
			}
			catch (Exception ex)
			{
				line = null;
				break;
			}
			if ( line == null ) break;
			if ( line.length () == 0 ) continue;
			if ( line.matches ("^\\s*#.*") ) continue;	// NOI18N

			allzeroM = allzeroPat.matcher (line);
			superM = superPat.matcher (line);
			bsizeM = bsizePat.matcher (line);
			forceM = forcePat.matcher (line);
			iterM = iterPat.matcher (line);
			lastzeroM = lastzeroPat.matcher (line);
			nopartM = nopartPat.matcher (line);
			nounrmM = nounrmPat.matcher (line);
			nowfsM = nowfsPat.matcher (line);
			ioctlM = ioctlPat.matcher (line);
			fsListM = fsListPat.matcher (line);
			xM = xPat.matcher (line);
			yM = yPat.matcher (line);
			widthM = widthPat.matcher (line);
			heightM = heightPat.matcher (line);
			isMaxM = isMaxPat.matcher (line);
			fontSizeM = fontSizePat.matcher (line);
			wfsPathM = wfsPathPat.matcher (line);
			isMethodM = isMethodPat.matcher (line);
			methodNameM = methodNamePat.matcher (line);

			if ( allzeroM.matches () )
			{
				try
				{
					allzeros = Integer.parseInt (allzeroM.group (1)) != 0;
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (all_zero)");	// NOI18N
				}
			}
			else if ( superM.matches () )
			{
				try
				{
					superOffset = Integer.parseInt (superM.group (1));
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (super_off)");	// NOI18N
				}
			}
			else if ( bsizeM.matches () )
			{
				try
				{
					blockSize = Integer.parseInt (bsizeM.group (1));
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (block_size)");	// NOI18N
				}
			}
			else if ( forceM.matches () )
			{
				try
				{
					force = Integer.parseInt (forceM.group (1)) != 0;
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (force)");	// NOI18N
				}
			}
			else if ( iterM.matches () )
			{
				try
				{
					iterations = Integer.parseInt (iterM.group (1));
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (interations)"); // NOI18N
				}
			}
			else if ( lastzeroM.matches () )
			{
				try
				{
					lastZero = Integer.parseInt (lastzeroM.group (1)) != 0;
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (last_zero)");	// NOI18N
				}
			}
			else if ( nopartM.matches () )
			{
				try
				{
					nopart = Integer.parseInt (nopartM.group (1)) != 0;
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (no_part)");	// NOI18N
				}
			}
			else if ( nounrmM.matches () )
			{
				try
				{
					nounrm = Integer.parseInt (nounrmM.group (1)) != 0;
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (no_unrm)");	// NOI18N
				}
			}
			else if ( nowfsM.matches () )
			{
				try
				{
					nowfs = Integer.parseInt (nowfsM.group (1)) != 0;
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (no_wfs)");	// NOI18N
				}
			}
			else if ( ioctlM.matches () )
			{
				try
				{
					ioctl = Integer.parseInt (ioctlM.group (1)) != 0;
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (ioctl)");	// NOI18N
				}
			}
			else if ( fsListM.matches () )
			{
				try
				{
					String[] fs = fsListM.group (1).split ("\000");	// NOI18N
					if ( fs != null )
					{
						for ( int i=0; i < fs.length; i++ )
						{
							if ( fs[i] == null ) continue;
							if ( fs[i].length () == 0 ) continue;
							fsList.add (fs[i]);
						}
					}
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (fs_list)");	// NOI18N
				}
			}
			else if ( xM.matches () )
			{
				try
				{
					x = Integer.parseInt (xM.group (1));
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (x)");	// NOI18N
				}
			}
			else if ( yM.matches () )
			{
				try
				{
					y = Integer.parseInt (yM.group (1));
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (y)");	// NOI18N
				}
			}
			else if ( widthM.matches () )
			{
				try
				{
					width = Integer.parseInt (widthM.group (1));
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (width)");	// NOI18N
				}
			}
			else if ( heightM.matches () )
			{
				try
				{
					height = Integer.parseInt (heightM.group (1));
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (height)");	// NOI18N
				}
			}
			else if ( isMaxM.matches () )
			{
				try
				{
					isMax = Integer.parseInt (isMaxM.group (1)) != 0;
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (is_max)");	// NOI18N
				}
			}
			else if ( fontSizeM.matches () )
			{
				try
				{
					fontSize = Integer.parseInt (fontSizeM.group (1));
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (font_size)");	// NOI18N
				}
			}
			else if ( wfsPathM.matches () )
			{
				wfsPath = wfsPathM.group (1);
			}
			else if ( isMethodM.matches () )
			{
				try
				{
					selectMethod = Integer.parseInt (isMethodM.group (1)) != 0;
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex, "ConfigFile.read.parseInt (is_method)");	// NOI18N
				}
			}
			else if ( methodNameM.matches () )
			{
				methodName = methodNameM.group (1);
			}
		} while (line != null);
		br.close ();

		// verify here
		if ( x < 0 ) x = 0;
		if ( y < 0 ) y = 0;
		if ( width < 0 ) width = 640;
		if ( height < 0 ) height = 480;
		if ( fontSize < 0 ) fontSize = 16;
	}

	/**
	 * Writes the parameters to the given file.
	 * @throws Exception on file error.
	 */
	public void write () throws Exception
	{
		String newLine = System.getProperty ("line.separator", "\n");			// NOI18N
		if ( newLine == null )
		{
			newLine = "\n";								// NOI18N
		}
		else if ( newLine.length () == 0 )
		{
			newLine = "\n";								// NOI18N
		}
		// don't force a specific encoding
		PrintWriter pw = new PrintWriter (cfgFile);
		StringBuilder fsString = new StringBuilder (100);
		String zero = "\000";								// NOI18N
		if ( fsList != null )
		{
			for ( int i=0; i < fsList.size (); i++ )
			{
				fsString.append (fsList.get (i) + zero);
			}
		}
		pw.println (
			  "all_zero = " + String.valueOf ((allzeros)? 1 : 0) + newLine		// NOI18N
			+ "superblock_offset = " + superOffset + newLine			// NOI18N
			+ "blocksize = " + blockSize + newLine					// NOI18N
			+ "force = " + String.valueOf ((force)? 1 : 0) + newLine		// NOI18N
			+ "iterations = " + iterations + newLine				// NOI18N
			+ "last_zero = " + String.valueOf ((lastZero)? 1 : 0) + newLine		// NOI18N
			+ "no_part = " + String.valueOf ((nopart)? 1 : 0) + newLine		// NOI18N
			+ "no_undel = " + String.valueOf ((nounrm)? 1 : 0) + newLine		// NOI18N
			+ "no_free = " + String.valueOf ((nowfs)? 1 : 0) + newLine		// NOI18N
			+ "no_cache = " + String.valueOf ((ioctl)? 1 : 0) + newLine		// NOI18N
			+ "x = " + x + newLine							// NOI18N
			+ "y = " + y + newLine							// NOI18N
			+ "width = " + width + newLine						// NOI18N
			+ "height = " + height + newLine					// NOI18N
			+ "ismax = " + String.valueOf ((isMax)? 1 : 0) + newLine		// NOI18N
			+ "filesystems = " + fsString.toString() + newLine			// NOI18N
			+ "font_size = " + fontSize + newLine					// NOI18N
			+ "wfs_path = " + wfsPath + newLine					// NOI18N
			+ "is_method = " + String.valueOf ((selectMethod)? 1 : 0) + newLine	// NOI18N
			+ "method = " + methodName + newLine					// NOI18N
		);
		pw.close ();
	}

	// ================ setters:

	/**
	 * Sets the X variable.
	 * @param v the new value.
	 */
	public void setX (int v)
	{
		x = v;
	}

	/**
	 * Sets the Y variable.
	 * @param v the new value.
	 */
	public void setY (int v)
	{
		y = v;
	}

	/**
	 * Sets the width variable.
	 * @param v the new value.
	 */
	public void setWidth (int v)
	{
		width = v;
	}

	/**
	 * Sets the height variable.
	 * @param v the new value.
	 */
	public void setHeight (int v)
	{
		height = v;
	}

	/**
	 * Sets the isMaximized variable.
	 * @param v the new value.
	 */
	public void setIsMaximized (boolean v)
	{
		isMax = v;
	}

	/**
	 * Sets the all-zeros variable.
	 * @param v the new value.
	 */
	public void setAllZeros (boolean v)
	{
		allzeros = v;
	}

	/**
	 * Sets the superblock offset variable.
	 * @param v the new value.
	 */
	public void setSuperOffset (int v)
	{
		superOffset = v;
	}

	/**
	 * Sets the block size variable.
	 * @param v the new value.
	 */
	public void setBlockSize (int v)
	{
		blockSize = v;
	}

	/**
	 * Sets the "force" variable.
	 * @param v the new value.
	 */
	public void setForce (boolean v)
	{
		force = v;
	}

	/**
	 * Sets the iterations variable.
	 * @param v the new value.
	 */
	public void setIterations (int v)
	{
		iterations = v;
	}

	/**
	 * Sets the last-zero variable.
	 * @param v the new value.
	 */
	public void setLastZero (boolean v)
	{
		lastZero = v;
	}

	/**
	 * Sets the wipe-partially-used-blocks variable.
	 * @param v the new value.
	 */
	public void setWipePart (boolean v)
	{
		nopart = v;
	}

	/**
	 * Sets the wipe-undelete variable.
	 * @param v the new value.
	 */
	public void setWipeUndel (boolean v)
	{
		nounrm = v;
	}

	/**
	 * Sets the wipe-free-space variable.
	 * @param v the new value.
	 */
	public void setWipeFreeSpace (boolean v)
	{
		nowfs = v;
	}

	/**
	 * Sets the ioctl variable.
	 * @param v the new value.
	 */
	public void setIoctl (boolean v)
	{
		ioctl = v;
	}

	/**
	 * Sets the fsList variable.
	 * @param v the new value.
	 */
	public void setFSList (Vector<String> v)
	{
		fsList = new Vector<String> (v);
	}

	/**
	 * Sets the font size variable.
	 * @param v the new value.
	 */
	public void setFontSizeValue (int v)
	{
		fontSize = v;
	}

	/**
	 * Sets the wipefreespace path variable.
	 * @param v the new value.
	 */
	public void setWfsPath (String v)
	{
		wfsPath = v;
	}

	/**
	 * Sets the selectMethod variable.
	 * @param v the new value.
	 */
	public void setIsMethodSelected (boolean v)
	{
		selectMethod = v;
	}

	/**
	 * Sets the method name variable.
	 * @param v the new value.
	 */
	public void setMethodName (String v)
	{
		methodName = v;
	}

	// ================ getters:

	/**
	 * Gets the X variable.
	 * @return the variable's value.
	 */
	public int getX ()
	{
		return x;
	}

	/**
	 * Gets the Y variable.
	 * @return the variable's value.
	 */
	public int getY ()
	{
		return y;
	}

	/**
	 * Gets the width variable.
	 * @return the variable's value.
	 */
	public int getWidth ()
	{
		return width;
	}

	/**
	 * Gets the height variable.
	 * @return the variable's value.
	 */
	public int getHeight ()
	{
		return height;
	}

	/**
	 * Gets the isMax variable.
	 * @return the variable's value.
	 */
	public boolean getIsMax ()
	{
		return isMax;
	}

	/**
	 * Gets the all-zeros variable.
	 * @return the variable's value.
	 */
	public boolean getAllZeros ()
	{
		return allzeros;
	}

	/**
	 * Gets the superblock offset variable.
	 * @return the variable's value.
	 */
	public int getSuperOffset ()
	{
		return superOffset;
	}

	/**
	 * Gets the block size variable.
	 * @return the variable's value.
	 */
	public int getBlockSize ()
	{
		return blockSize;
	}

	/**
	 * Gets the "force" variable.
	 * @return the variable's value.
	 */
	public boolean getForce ()
	{
		return force;
	}

	/**
	 * Gets the iterations variable.
	 * @return the variable's value.
	 */
	public int getIterations ()
	{
		return iterations;
	}

	/**
	 * Gets the last-zero variable.
	 * @return the variable's value.
	 */
	public boolean getLastZero ()
	{
		return lastZero;
	}

	/**
	 * Gets the wipe-partially-used-blocks variable.
	 * @return the variable's value.
	 */
	public boolean getWipePart ()
	{
		return nopart;
	}

	/**
	 * Gets the wipe-undelete variable.
	 * @return the variable's value.
	 */
	public boolean getWipeUndel ()
	{
		return nounrm;
	}

	/**
	 * Gets the wipe-free-space variable.
	 * @return the variable's value.
	 */
	public boolean getWipeFreeSpace ()
	{
		return nowfs;
	}

	/**
	 * Gets the ioctl variable.
	 * @return the variable's value.
	 */
	public boolean getIoctl ()
	{
		return ioctl;
	}

	/**
	 * Gets the fsList variable.
	 * @return the variable's value.
	 */
	public Vector<String> getFSList ()
	{
		return fsList;
	}

	/**
	 * Gets the font size variable.
	 * @return the variable's value.
	 */
	public int getFontSizeValue ()
	{
		return fontSize;
	}

	/**
	 * Gets the wipefreespace path variable.
	 * @return the variable's value.
	 */
	public String getWfsPath ()
	{
		return wfsPath;
	}

	/**
	 * Gets the selectMethod variable.
	 * @return the variable's value.
	 */
	public boolean getIsMethod ()
	{
		return selectMethod;
	}

	/**
	 * Gets the method name variable.
	 * @return the variable's value.
	 */
	public String getMethodName ()
	{
		return methodName;
	}
}
