/*
 * WipeFreeSpaceGUI.java, part of the WipeFreeSpaceGUI package.
 *
 * Copyright (C) 2009-2012 Bogdan Drozdowski, bogdandr (at) op.pl
 * License: GNU General Public License, v3+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foudation:
 *		Free Software Foundation
 *		51 Franklin Street, Fifth Floor
 *		Boston, MA 02110-1301
 *		USA
 *
 */

package BogDroSoft.wfs;

import java.awt.Component;
import java.awt.Container;
import java.awt.Window;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.PrintStream;
import java.util.Arrays;
import java.util.Calendar;
import java.util.EventObject;
import javax.swing.JFrame;
import javax.swing.SwingUtilities;
import org.jdesktop.application.Application;
import org.jdesktop.application.SingleFrameApplication;

/**
 * The main class of the application.
 */
public class WipeFreeSpaceGUI extends SingleFrameApplication
{
	private static String[] argv;
	private static String logFile = "wfsgui.log";	// complete name: path + filename	// NOI18N
	private static final String logFileName = "wfsgui.log"; // the non-changeable filename	// NOI18N

	/**
	 * At startup create and show the main frame of the application.
	 */
	@Override
	protected void startup ()
	{
		getContext ().getLocalStorage ().setStorageLimit (0);
		show (new WipeFreeSpaceGUIView (this));
	}

	/**
	 * This method is to initialize the specified window by injecting resources.
	 * Windows shown in our application come fully initialized from the GUI
	 * builder, so this additional configuration is not needed.
	 * @param root The root window.
	 */
	@Override
	protected void configureWindow (java.awt.Window root) {}

	/**
	 * A convenient static getter for the application instance.
	 * @return the instance of WipeFreeSpaceGUI
	 */
	public static WipeFreeSpaceGUI getApplication ()
	{
		WipeFreeSpaceGUI app = Application.getInstance (WipeFreeSpaceGUI.class);
		app.getContext ().getLocalStorage ().setStorageLimit (0);
		return app;
	}

	/**
	 * Exit the application.
	 * @param eo The EventObject that triggerred the exit (can be null).
	 */
	@Override
	public void exit (EventObject eo)
	{
		// do NOT call super.exit() - that method
		// saves the session data to a directory in user's home.
		ExitListener[] listeners = getExitListeners ();
		if ( listeners != null )
		{
			// first check if any listener objects against exiting:
			for ( int i = 0; i < listeners.length; i++ )
			{
				if ( listeners[i] == null ) continue;
				if ( ! listeners[i].canExit (eo) ) return;
			}
			// now notify every listener that we are about to exit:
			for ( int i = 0; i < listeners.length; i++ )
			{
				if ( listeners[i] != null ) listeners[i].willExit (eo);
			}
		}
		JFrame main = getMainFrame ();
		if ( main != null )
		{
			main.setVisible (false);
			main.dispose ();
		}
		end ();
	}

	/**
	 * Overridden to avoid writing "session data" to a directory in the user's
	 * home directory without permission.
	 */
	@Override
	protected void shutdown ()
	{
		System.exit (0);
	}

	/**
	 * Overridden to avoid writing "session data" to a directory in the user's
	 * home directory without permission.
	 */
	@Override
	protected void end ()
	{
		System.exit (0);
	}

	/**
	 * Displays all the important information about exceptions.
	 * @param ex The exception to display.
	 * @param data Any additional data to display.
	 */
	public static void handleException (Throwable ex, Object data)
	{
		if ( ex == null || (System.out == null && System.err == null) ) return;
		try
		{
			Calendar c = Calendar.getInstance ();

			int month  = c.get (Calendar.MONTH);
			int day    = c.get (Calendar.DAY_OF_MONTH);
			int hour   = c.get (Calendar.HOUR_OF_DAY);
			int minute = c.get (Calendar.MINUTE);
			int second = c.get (Calendar.SECOND);
			     if ( month == Calendar.JANUARY   ) month =  1;
			else if ( month == Calendar.FEBRUARY  ) month =  2;
			else if ( month == Calendar.MARCH     ) month =  3;
			else if ( month == Calendar.APRIL     ) month =  4;
			else if ( month == Calendar.MAY       ) month =  5;
			else if ( month == Calendar.JUNE      ) month =  6;
			else if ( month == Calendar.JULY      ) month =  7;
			else if ( month == Calendar.AUGUST    ) month =  8;
			else if ( month == Calendar.SEPTEMBER ) month =  9;
			else if ( month == Calendar.OCTOBER   ) month = 10;
			else if ( month == Calendar.NOVEMBER  ) month = 11;
			else if ( month == Calendar.DECEMBER  ) month = 12;

			String time = c.get (Calendar.YEAR) + "-"		// NOI18N
				+ ((month<10)?  "0" : "" ) + month  + "-"	// NOI18N
				+ ((day<10)?    "0" : "" ) + day    + " "	// NOI18N
				+ ((hour<10)?   "0" : "" ) + hour   + ":"	// NOI18N
				+ ((minute<10)? "0" : "" ) + minute + ":"	// NOI18N
				+ ((second<10)? "0" : "" ) + second + ": ";	// NOI18N

			if ( System.out != null )
			{
				System.out.print (time + ex);
				System.out.flush ();
			}
			if ( System.err != null )
			{
				System.err.print (time + ex);
				System.err.flush ();
			}

		} catch (Throwable e) {}

		try
		{
			String msg = ex.getMessage ();
			if ( msg != null )
			{
				if ( System.out != null )
				{
					System.out.print (", Message='" + msg + "'");	// NOI18N
					System.out.flush ();
				}
				if ( System.err != null )
				{
					System.err.print (", Message='" + msg + "'");	// NOI18N
					System.err.flush ();
				}
			}
		} catch (Throwable e) {}

		try
		{
			if ( data != null )
			{
				if ( System.out != null )
				{
					System.out.print (", Data='" + data.toString () + "'");	// NOI18N
					System.out.flush ();
				}
				if ( System.err != null )
				{
					System.err.print (", Data='" + data.toString () + "'");	// NOI18N
					System.err.flush ();
				}
			}
		} catch (Throwable e) {}

		try
		{
			if ( System.out != null )
			{
				System.out.println ();	// NOI18N
				System.out.flush ();
			}
			if ( System.err != null )
			{
				System.err.println ();	// NOI18N
				System.err.flush ();
			}
		} catch (Throwable e) {}

		StackTraceElement[] ste = ex.getStackTrace ();
		if ( ste != null )
		{
			for ( int i=0; i < ste.length; i++ )
			{
				if ( ste[i] != null )
				{
					try
					{
						String clazz = ste[i].getClassName ();
						String file = ste[i].getFileName ();
						String function = ste[i].getMethodName ();
						int line = ste[i].getLineNumber ();
						String toShow = "\tat\t";	// NOI18N
						if ( clazz != null )
						{
							// let's display only our files
							if ( ! clazz.startsWith
								(WipeFreeSpaceGUI.class.getPackage ().getName ()) )
							{
								continue;
							}
							toShow += clazz;
						}
						else
						{
							toShow += "<Unknown class>";	// NOI18N
						}
						if ( function != null )
						{
							toShow += "." + function;	// NOI18N
						}
						else
						{
							toShow += ".<Unknown method>";	// NOI18N
						}
						if ( file != null )
						{
							toShow += " (" + file;	// NOI18N
						}
						else
						{
							toShow += " (<Unknown file>";	// NOI18N
						}
						toShow += ":" + String.valueOf (line) + ")";	// NOI18N

						if ( System.out != null )
						{
							System.out.println (toShow);
							System.out.flush ();
						}
						if ( System.err != null )
						{
							System.err.println (toShow);
							System.err.flush ();
						}
					} catch (Throwable e) {}
				}
			}
		}
	}

	/**
	 * Returns the program's command-line arguments.
	 * @return the program's command-line arguments.
	 */
	public String[] getArgs ()
	{
		return Arrays.asList (argv).toArray(new String[]{});
	}

	/**
	 * Returns the name of the log file in use.
	 * @return the name of the log file in use.
	 */
	public String getLogfileName ()
	{
        	return logFile;
	}

	/**
	 * Method used to change the GUI - runs r.run () on the
	 * EventDispatchThread and waits for it to exit.
	 * @param r The code to run.
	 */
	public synchronized static void changeGUI (final Runnable r)
	{
		if ( r == null ) return;
		if ( SwingUtilities.isEventDispatchThread () )
		{
			try
			{
				r.run ();
			}
			catch (Throwable ex)
			{
				handleException (ex, "changeGUI->r.run");	// NOI18N
			}
		}
		else
		{
			try
			{
				SwingUtilities.invokeAndWait (new Runnable ()
				{
					@Override
					public synchronized void run ()
					{
						try
						{
							r.run ();
						}
						catch (Throwable ex)
						{
							handleException (ex,
								"changeGUI->invokeAndWait->r.run");// NOI18N
						}
					}
				});
			}
			catch (InterruptedException ex)
			{
				// can be thrown when closing the program, so ignore
				//handleException (ex, "changeGUI->invokeAndWait->Interrupt");// NOI18N
			}
			catch (Throwable ex)
			{
				handleException (ex, "changeGUI->invokeAndWait");	// NOI18N
			}
		}
	}

	/**
	 * Set the given font size in all the components in the given
	 * Component (recursively, if it's a Container).
	 * @param c The Component with Components that will have their font
	 *	size changed.
	 * @param newSize The new font size to set.
	 */
	public static void setFontSize (Component c, float newSize)
	{
		if ( c == null ) return;
		c.setFont (c.getFont ().deriveFont (newSize));
		if ( c instanceof Container )
		{
			Component[] subComps = ((Container)c).getComponents ();
			if ( subComps != null )
			{
				for ( int i = 0; i < subComps.length; i++ )
				{
					if ( subComps[i] != null )
						setFontSize (subComps[i], newSize);
				}
			}
		}
	}

	/**
	 * Main method launching the application.
	 * @param args Command-line arguments (currently unused).
	 */
	/*public*/ static void main (String[] args)
	{
		argv = args;

		// set default uncaught exception handler:
		Thread.setDefaultUncaughtExceptionHandler (handler);

		// redirect stderr (caught and uncaught exceptions) to a file
		try
		{
			// don't force a specific encoding, because the messages
			// might have been translated
			System.setErr (new PrintStream (new File (logFile)));
		}
		catch (Exception ex)
		{
			String[] dirs = new String[7];
			String filesep = "/";	// NOI18N
			boolean success = false;
			try
			{
				 filesep = System.getProperty ("file.separator", "/");	// NOI18N
			} catch (Exception exc) {}
			try
			{
				 dirs[0] = System.getenv ("TEMP");	// NOI18N
			} catch (Exception exc) {}
			try
			{
				 dirs[1] = System.getenv ("TEMPDIR");	// NOI18N
			} catch (Exception exc) {}
			try
			{
				 dirs[2] = System.getenv ("TMP");	// NOI18N
			} catch (Exception exc) {}
			try
			{
				 dirs[3] = System.getenv ("TMPDIR");	// NOI18N
			} catch (Exception exc) {}
			try
			{
				 dirs[4] = System.getenv ("HOME");	// NOI18N
			} catch (Exception exc) {}
			try
			{
				 dirs[5] = System.getProperty ("user.home");	// NOI18N
			} catch (Exception exc) {}
			try
			{
				 dirs[6] = System.getProperty ("user.dir");	// NOI18N
			} catch (Exception exc) {}
			for ( int i=0; i < dirs.length; i++ )
			{
				if ( dirs[i] == null ) continue;
				logFile = dirs[i] + filesep + logFileName;
				try
				{
					// don't force a specific encoding, because the messages
					// might have been translated
					System.setErr (new PrintStream (new File (logFile)));
					success = true;
					break;
				} catch (Exception exc) {}
			}
			if ( ! success ) handleException (ex, "logFile");	// NOI18N
		}
		launch (WipeFreeSpaceGUI.class, args);
	}

	/**
	 * A class that closes the window when the Esc key is pressed. Must
	 * be registered on all elements of the window that are desired to
	 * catch this key.
	 */
	public static class EscKeyListener extends KeyAdapter
	{
		private Window frame; // need Window for dispose().

		/**
		 * Creates an EscKeyListener for the given Window.
		 * @param wFrame The frame to connect this listener to. All
		 *	of the frame's Components will have this listener
		 *	installed, too.
		 */
		public EscKeyListener (Window wFrame)
		{
			frame = wFrame;
			addKeyListeners (frame);
		}

		/**
		 * Recursively adds this keylistener to the given Component
		 *	and all its subcomponents.
		 * @param c The Component with Components that will have this keylistener.
		 */
		private void addKeyListeners (Component c)
		{
			if ( c == null ) return;
			c.addKeyListener (this);
			if ( c instanceof Container )
			{
				Component[] subComps = ((Container)c).getComponents ();
				if ( subComps != null )
				{
					for ( int i = 0; i < subComps.length; i++ )
					{
						if ( subComps[i] != null )
							addKeyListeners (subComps[i]);
					}
				}
			}
		}

		/**
		 * Receives key-typed events (called when the user types a key).
		 * @param ke The key-typed event.
		 */
		@Override
		public void keyTyped (KeyEvent ke)
		{
			if ( ke == null || frame == null ) return;
			if ( ke.getKeyChar () == KeyEvent.VK_ESCAPE )
			{
				frame.dispose ();
			}
		}
	}

	/**
	 * An uncaught-exception handler instance for WipeFreeSpaceGUI.
	 */
	public static final UncExHndlr handler = new UncExHndlr ();
	/**
	 * A simple uncaught-exception handler class for threads.
	 */
	public static class UncExHndlr implements Thread.UncaughtExceptionHandler
	{
		/**
		 * Called when an uncaught exception occurrs.
		 * @param t The thread, in which the exception occurred.
		 * @param ex The exception that occurred.
		 */
		@Override
		public void uncaughtException (Thread t, Throwable ex)
		{
			try
			{
				handleException (ex, "WipeFreeSpaceGUI.UncaughtExceptionHandler: Thread="	// NOI18N
					+ ((t != null)? t.getName() : "?"));	// NOI18N
			} catch (Throwable th) {}
		}
	}
}
