/*
 * WipeFreeSpaceGUIView.java, part of the WipeFreeSpaceGUI package.
 *
 * Copyright (C) 2009-2012 Bogdan Drozdowski, bogdandr (at) op.pl
 * License: GNU General Public License, v3+
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 3
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foudation:
 *		Free Software Foundation
 *		51 Franklin Street, Fifth Floor
 *		Boston, MA 02110-1301
 *		USA
 *
 */

package BogDroSoft.wfs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.awt.event.ItemEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.EventObject;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.ListModel;
import javax.swing.SwingWorker;
import org.jdesktop.application.Action;
import org.jdesktop.application.Application.ExitListener;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;

/**
 * The application's main frame.
 */
public class WipeFreeSpaceGUIView extends FrameView
{
	private JDialog aboutBox = null;
	private JFrame mainFrame;
	/** The current version number as a String. */
	public static final String verString = "0.5";	// NOI18N
	private final WipeFreeSpaceGUI app = WipeFreeSpaceGUI.getApplication ();
	private JFileChooser fsChooser;
	private JFileChooser cfgFC;
	private JFileChooser progFC;

	private volatile Process wfs = null;

	//use a SwingWorker just to make jlint happy:
	//private Thread wfs_monitor = null;
	private SwingWorker<Void, Void> wfs_monitor = null;
	private ProgressUpdater stdoutUpdater, stderrUpdater;
	private static final ResourceBundle gbundle =
		ResourceBundle.getBundle("BogDroSoft/wfs/resources/WipeFreeSpaceGUIView");

	// i18n stuff:
	private static final String questionTitle  = gbundle.getString("Question");
	private static final String errorTitle     = gbundle.getString("Error");
	private static final String stopProcessMsg = gbundle.getString("Really_stop");
	private static final String noFSMsg        = gbundle.getString("No_filesystems");
	private static final String cantExecMsg    = gbundle.getString("cant_start");
	private static final String wfsNotFoundMsg = gbundle.getString("wipefreespace_not_found");
	private static final String progIntroStr   = gbundle.getString("A_GUI_for_WipeFreeSpace");
	private static final String cmdLineStr     = gbundle.getString("Commandline_options")
		+ ":" +	// NOI18N
		"\n--all-zeros\t\t- " +	// NOI18N
		gbundle.getString("wipe_only_using_zeros") +
		"\n-b|--superblock <off>\t- " +	// NOI18N
		gbundle.getString("superblock_offset") +
		"\n-B|--blocksize <size>\t- " +	// NOI18N
		gbundle.getString("block_size") +
		"\n--conf <file>\t\t- " +	// NOI18N
		gbundle.getString("config_from_file") +
		"\n-f|--force\t\t- " +	// NOI18N
		gbundle.getString("continue_if_errors") +
		"\n--help, -h, -?, /?\t- "+	// NOI18N
		gbundle.getString("print_help") +
		"\n--lang LL_CC_VV\t\t- "+	// NOI18N
		gbundle.getString("select_language") +
		":\n\t\t\t  " +	// NOI18N
		gbundle.getString("LL_language_CC") +
		"\n\t\t\t  " +	// NOI18N
		gbundle.getString("country_code_VV_variant") +
		".\n\t\t\t  " +	// NOI18N
		gbundle.getString("Separate_them_LL_required") +
		"\n--last-zero\t\t- " +	// NOI18N
		gbundle.getString("add_wiping_with_zeros") +
		"\n--licence, --license\t- "+	// NOI18N
		gbundle.getString("display_license") +
		"\n--method <name>\t\t- " +	// NOI18N
		gbundle.getString("help_wiping_method") +
		"\n-n|--iterations <n>\t- " +	// NOI18N
		gbundle.getString("number_of_iterations") +
		"\n--nopart\t\t- " +	// NOI18N
		gbundle.getString("do_NOT_wipe_part") +
		"\n--nounrm\t\t- " +	// NOI18N
		gbundle.getString("do_NOT_wipe_undel") +
		"\n--nowfs\t\t\t- " +	// NOI18N
		gbundle.getString("do_NOT_wipe_unused") +
		"\n--use-ioctl\t\t- " +	// NOI18N
		gbundle.getString("disable_cache") +
		"\n--version, -V\t\t- "+	// NOI18N
		gbundle.getString("display_version")
		;
	private static final String verWord = gbundle.getString("version");
	private static final String WFSConfFiles = gbundle.getString("WipeFreeSpace_configuration_files");
	private static final String exOverString = gbundle.getString("exists_Overwrite");
	private static final String fileNotWriteMsg = gbundle.getString("Cant_write_to_file");
	private static final String noWorkMsg = gbundle.getString("NOTHING_TO_WIPE");

	/**
	 * Creates a new WipeFreeSpaceGUIView, which is the main window of the
	 * WipeFreeSpaceGUI program.
	 * @param sfapp The parent application to get data from.
	 */
	public WipeFreeSpaceGUIView (final SingleFrameApplication sfapp)
	{
		super (sfapp);
		// set uncaught exception handler for GUI threads, just in case:
		try
		{
			Thread[] ths = new Thread[Thread.activeCount () * 5];
			if ( ths != null )
			{
				final int nThreads = Thread.enumerate (ths);
				for ( int i=0; i < nThreads; i++ )
				{
					String name = ths[i].getName ();
					if ( name == null ) continue;
					if ( name.contains ("AWT") // NOI18N
						|| name.contains ("Swing") // NOI18N
						|| name.contains ("Image") // NOI18N
					   )
					{
						ths[i].setUncaughtExceptionHandler (WipeFreeSpaceGUI.handler);
					}
				}
			}
		} catch (Throwable th) {}

		// do '--help', '--license' and '--version' first, without opening the GUI
		String[] args = app.getArgs ();
		if ( args != null )
		{
			for ( int i=0; i < args.length; i++ )
			{
				if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--help")	// NOI18N
					|| args[i].toLowerCase (Locale.ENGLISH).equals ("-h")	// NOI18N
					|| args[i].toLowerCase (Locale.ENGLISH).equals ("-?")	// NOI18N
					|| args[i].toLowerCase (Locale.ENGLISH).equals ("/?") )	// NOI18N
				{
					System.out.println ("WipeFreeSpaceGUI - "+ progIntroStr +	// NOI18N
						"\nAuthor: Bogdan Drozdowski, bogdandr @ op . pl\n" +	// NOI18N
						"License: GPLv3+\n" +	// NOI18N
						"http://rudy.mif.pg.gda.pl/~bogdro/soft\n\n" +	// NOI18N
						cmdLineStr
						);
					app.exit (null);
				}
				else if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--license")	// NOI18N
					|| args[i].toLowerCase (Locale.ENGLISH).equals ("--licence") )	// NOI18N
				{
					System.out.println ("WipeFreeSpaceGUI - "+ progIntroStr +	// NOI18N
						"\nSee http://rudy.mif.pg.gda.pl/~bogdro/soft\n" +	// NOI18N
						"Author: Bogdan 'bogdro' Drozdowski, bogdandr @ op . pl.\n\n" +	// NOI18N
						"    This program is free software; you can redistribute it and/or\n" +	// NOI18N
						"    modify it under the terms of the GNU General Public License\n" +	// NOI18N
						"    as published by the Free Software Foundation; either version 3\n" +	// NOI18N
						"    of the License, or (at your option) any later version.\n\n" +	// NOI18N
						"    This program is distributed in the hope that it will be useful,\n" +	// NOI18N
						"    but WITHOUT ANY WARRANTY; without even the implied warranty of\n" +	// NOI18N
						"    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n" +	// NOI18N
						"    GNU General Public License for more details.\n\n" +	// NOI18N
						"    You should have received a copy of the GNU General Public License\n" +	// NOI18N
						"    along with this program; if not, write to the Free Software Foundation:\n" +	// NOI18N
						"               Free Software Foundation\n" +	// NOI18N
						"               51 Franklin Street, Fifth Floor\n" +	// NOI18N
						"               Boston, MA 02110-1301\n" +	// NOI18N
						"               USA\n");	// NOI18N
					app.exit (null);
				}
				else if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--version")	// NOI18N
					|| args[i].equals ("-V") )	// NOI18N
				{
					System.out.println ("WipeFreeSpaceGUI " + verWord + " " + verString);	// NOI18N
					app.exit (null);
				}
			} // for i (args)
		} // args != null
		initComponents ();
		ioctlCb.setForeground (new Color (0, 204, 0));
		forceCb.setForeground (new Color (0, 204, 0));
		mainFrame = getFrame ();//sfapp.getMainFrame (); <- this does NOT work
		mainFrame.setTitle ("WipeFreeSpace GUI " + verString);	// NOI18N
		// set the minimum size a bit bigger than the preferred size, because of the scroll bars:
		Dimension prefSize = contentPanel.getPreferredSize ();
		//mainFrame.setMinimumSize (new Dimension (prefSize.width + 50, prefSize.height + 50));
		mainFrame.setPreferredSize (new Dimension (prefSize.width + 50, prefSize.height + 50));
//		mainFrame.setPreferredSize (contentPanel.getPreferredSize ());
//		app.getMainView ().getFrame ().setMinimumSize (contentPanel.getPreferredSize ());
//		app.getMainView ().getFrame ().setPreferredSize (contentPanel.getPreferredSize ());
//		app.getMainView ().getRootPane ().setMinimumSize (contentPanel.getPreferredSize ());
//		app.getMainView ().getRootPane ().setPreferredSize (contentPanel.getPreferredSize ());
		fontSizeSpin.setValue (fontSizeSpin.getValue ());	// refresh the font in the window

		if ( args != null )
		{
			for ( int i=0; i < args.length; i++ )
			{
				if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--all-zeros") )	// NOI18N
				{
					onlyZerosCb.setSelected (true);
				}
				else if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--superblock")	// NOI18N
					|| args[i].equals ("-b") )	// NOI18N
				{
					superOffCb.setSelected (true);
					if ( i+1 < args.length ) superOffText.setText (args[i+1]);
					i++;
				}
				else if ( args[i].equals ("-B")	// NOI18N
					|| args[i].toLowerCase (Locale.ENGLISH).equals ("--blocksize") )	// NOI18N
				{
					blkSizeCb.setSelected (true);
					if ( i+1 < args.length ) blkSizeText.setText (args[i+1]);
					i++;
				}
				else if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--conf") )	// NOI18N
				{
					if ( i+1 < args.length )
					{
						// have to wait until the GUI is initialized:
						final String cfgFileName = args[i+1];
						SwingWorker<Void, Void> sw = new SwingWorker<Void, Void> ()
						{
							@Override
							protected Void doInBackground ()
							{
								while ( mainFrame.getFont () == null )
								{
									try {Thread.sleep (10);}
									catch (Exception ex) {}
								}
								return null;
							}

							@Override
							protected void done ()
							{
								readConfig (new File (cfgFileName));
							}
						};
						sw.execute ();
						i++;
					}
				}
				else if ( args[i].equals ("-f")	// NOI18N
					|| args[i].toLowerCase (Locale.ENGLISH).equals ("--force") )	// NOI18N
				{
					forceCb.setSelected (true);
				}
				else if ( args[i].toLowerCase ().equals ("--lang") )	// NOI18N
				{
					if ( i < args.length-1 )
					{
						try
						{
							String[] locale = args[i+1].split ("_");	// NOI18N
							if ( locale != null )
							{
								Locale newLoc = null;
								if (locale.length == 1)
									newLoc = new Locale (locale[0]);
								else if (locale.length == 2)
									newLoc = new Locale (locale[0],
										locale[1]);
								else if (locale.length == 3)
									newLoc = new Locale (locale[0],
										locale[1], locale[2]);
								if ( newLoc != null )
									Locale.setDefault (newLoc);
							}
						}
						catch ( Exception ex )
						{
							WipeFreeSpaceGUI.handleException (ex,
								"main.lang(" + args[i+1] + ")");	// NOI18N
						}
						i++;
					}
				}
				else if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--last-zero") )	// NOI18N
				{
					lastZeroCb.setSelected (true);
				}
				else if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--method") )	// NOI18N
				{
					methodCb.setSelected (true);
					if ( i+1 < args.length ) methodList.setSelectedItem (args[i+1]);
					i++;
				}
				else if ( args[i].equals ("-n")	// NOI18N
					|| args[i].toLowerCase (Locale.ENGLISH).equals ("--iterations") )	// NOI18N
				{
					iterCb.setSelected (true);
					if ( i+1 < args.length ) iterText.setText (args[i+1]);
					i++;
				}
				else if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--nopart") )	// NOI18N
				{
					nopartCb.setSelected (true);
				}
				else if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--nounrm") )	// NOI18N
				{
					nounrmCb.setSelected (true);
				}
				else if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--nowfs") )	// NOI18N
				{
					nowfsCb.setSelected (true);
				}
				else if ( args[i].toLowerCase (Locale.ENGLISH).equals ("--use-ioctl") )	// NOI18N
				{
					ioctlCb.setSelected (true);
				}
			}	// for i (args)
		} // args != null

		app.addExitListener (new ExitListener ()
		{
			@Override
			public boolean canExit (EventObject ev)
			{
				if ( wfs != null )
				{
					try
					{
						int res = JOptionPane.showConfirmDialog (mainFrame,
							stopProcessMsg, questionTitle,
							JOptionPane.YES_NO_CANCEL_OPTION);
						if ( res != JOptionPane.YES_OPTION ) return false;
					} catch (Exception ex) {}
				}
				return true;
			}

			@Override
			public void willExit (EventObject ev)
			{
				// stop wfs and the threads, close the logfile
				if ( wfs != null )
				{
					// kill the process.
					try
					{
						// stop the Threads reading stdout and stderr
						if ( stdoutUpdater != null ) stdoutUpdater.stop ();
						if ( stderrUpdater != null ) stderrUpdater.stop ();
						// stop the process:
						wfs.destroy ();
						wfs = null;
						if ( wfs_monitor != null ) wfs_monitor.cancel (true);//.interrupt ();
					}
					catch (Exception ex)
					{
						WipeFreeSpaceGUI.handleException (ex,
							"willExit: stop");	// NOI18N
					}
				}
				// close the logfile - if it is empty, it can be deleted.
				if ( System.err != null ) System.err.close ();
				try
				{
					File log = new File (app.getLogfileName ());
					if ( log.exists () && log.length () == 0L )
					{
						boolean canDelete = log.delete ();
						if ( ! canDelete )
						{
							throw new Exception ("willExit: can't delete log file");	// NOI18N
						}
					}
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex,
						"willExit: log");	// NOI18N
				}
			}
		});

		/* add the Esc key listener to the frame and all components. */
		new WipeFreeSpaceGUI.EscKeyListener (mainFrame);
	}

	/**
	 * The Action that shows the "About" box. Has to be public.
	 */
	@Action
	public void showAboutBox ()
	{
		//if ( aboutBox == null )	// always new, to reset the state
		{
			aboutBox = new WipeFreeSpaceGUIAboutBox (mainFrame);
			aboutBox.setLocationRelativeTo (mainFrame);
		}
		//app.show (aboutBox);	// this makes the program save data in user's
								// home directory without permission
		aboutBox.setVisible (true);
	}

	/**
	 * This method is called from within the constructor to
	 * initialize the form.
	 * WARNING: Do NOT modify this code. The content of this method is
	 * always regenerated by the Form Editor.
	 */
	@SuppressWarnings("unchecked")
        // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
        private void initComponents() {

                mainPanel = new javax.swing.JPanel();
                contentScrollPane = new javax.swing.JScrollPane();
                contentPanel = new javax.swing.JPanel();
                jScrollPane1 = new javax.swing.JScrollPane();
                progOutput = new javax.swing.JTextArea();
                blkSizeCb = new javax.swing.JCheckBox();
                fontSizeSpin = new javax.swing.JSpinner();
                totProgLab = new javax.swing.JLabel();
                saveConfBut = new javax.swing.JButton();
                fsListLab = new javax.swing.JLabel();
                aboutBut = new javax.swing.JButton();
                fsProgBar = new javax.swing.JProgressBar();
                nounrmCb = new javax.swing.JCheckBox();
                stageProgLab = new javax.swing.JLabel();
                addBut = new javax.swing.JButton();
                errorLab = new javax.swing.JLabel();
                superOffCb = new javax.swing.JCheckBox();
                outputLab = new javax.swing.JLabel();
                fsProgLab = new javax.swing.JLabel();
                currFSLab = new javax.swing.JLabel();
                superOffText = new javax.swing.JTextField();
                totProgBar = new javax.swing.JProgressBar();
                jScrollPane3 = new javax.swing.JScrollPane();
                fsList = new javax.swing.JList();
                blkSizeText = new javax.swing.JTextField();
                onlyZerosCb = new javax.swing.JCheckBox();
                nowWipingLab = new javax.swing.JLabel();
                exitBut = new javax.swing.JButton();
                jScrollPane2 = new javax.swing.JScrollPane();
                progErrors = new javax.swing.JTextArea();
                startBut = new javax.swing.JButton();
                ioctlCb = new javax.swing.JCheckBox();
                iterCb = new javax.swing.JCheckBox();
                forceCb = new javax.swing.JCheckBox();
                nowfsCb = new javax.swing.JCheckBox();
                iterText = new javax.swing.JTextField();
                stageProgBar = new javax.swing.JProgressBar();
                stopBut = new javax.swing.JButton();
                loadConfBut = new javax.swing.JButton();
                nopartCb = new javax.swing.JCheckBox();
                fontSizeLab = new javax.swing.JLabel();
                optLabel = new javax.swing.JLabel();
                lastZeroCb = new javax.swing.JCheckBox();
                delBut = new javax.swing.JButton();
                pathLabel = new javax.swing.JLabel();
                pathField = new javax.swing.JTextField();
                methodCb = new javax.swing.JCheckBox();
                methodList = new javax.swing.JComboBox();
                wfsBrowseBut = new javax.swing.JButton();

                mainPanel.setName("mainPanel"); // NOI18N

                contentScrollPane.setName("contentScrollPane"); // NOI18N

                contentPanel.setName("contentPanel"); // NOI18N

                jScrollPane1.setName("jScrollPane1"); // NOI18N

                progOutput.setColumns(20);
                progOutput.setEditable(false);
                progOutput.setRows(5);
                progOutput.setName("progOutput"); // NOI18N
                jScrollPane1.setViewportView(progOutput);

                org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(BogDroSoft.wfs.WipeFreeSpaceGUI.class).getContext().getResourceMap(WipeFreeSpaceGUIView.class);
                blkSizeCb.setText(resourceMap.getString("blkSizeCb.text")); // NOI18N
                blkSizeCb.setName("blkSizeCb"); // NOI18N
                blkSizeCb.addChangeListener(new javax.swing.event.ChangeListener() {
                        public void stateChanged(javax.swing.event.ChangeEvent evt) {
                                blkSizeCbStateChanged(evt);
                        }
                });
                blkSizeCb.addItemListener(new java.awt.event.ItemListener() {
                        public void itemStateChanged(java.awt.event.ItemEvent evt) {
                                blkSizeCbItemStateChanged(evt);
                        }
                });

                fontSizeSpin.setModel(new javax.swing.SpinnerNumberModel(Integer.valueOf(12), Integer.valueOf(1), null, Integer.valueOf(1)));
                fontSizeSpin.setName("fontSizeSpin"); // NOI18N
                fontSizeSpin.addChangeListener(new javax.swing.event.ChangeListener() {
                        public void stateChanged(javax.swing.event.ChangeEvent evt) {
                                fontSizeSpinStateChanged(evt);
                        }
                });

                totProgLab.setText(resourceMap.getString("totProgLab.text")); // NOI18N
                totProgLab.setName("totProgLab"); // NOI18N

                saveConfBut.setIcon(resourceMap.getIcon("saveConfBut.icon")); // NOI18N
                saveConfBut.setText(resourceMap.getString("saveConfBut.text")); // NOI18N
                saveConfBut.setName("saveConfBut"); // NOI18N
                saveConfBut.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                saveConfButActionPerformed(evt);
                        }
                });

                fsListLab.setText(resourceMap.getString("fsListLab.text")); // NOI18N
                fsListLab.setName("fsListLab"); // NOI18N

                javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(BogDroSoft.wfs.WipeFreeSpaceGUI.class).getContext().getActionMap(WipeFreeSpaceGUIView.class, this);
                aboutBut.setAction(actionMap.get("showAboutBox")); // NOI18N
                aboutBut.setIcon(resourceMap.getIcon("aboutBut.icon")); // NOI18N
                aboutBut.setText(resourceMap.getString("aboutBut.text")); // NOI18N
                aboutBut.setName("aboutBut"); // NOI18N

                fsProgBar.setName("fsProgBar"); // NOI18N

                nounrmCb.setText(resourceMap.getString("nounrmCb.text")); // NOI18N
                nounrmCb.setName("nounrmCb"); // NOI18N
                nounrmCb.addChangeListener(new javax.swing.event.ChangeListener() {
                        public void stateChanged(javax.swing.event.ChangeEvent evt) {
                                nounrmCbStateChanged(evt);
                        }
                });
                nounrmCb.addItemListener(new java.awt.event.ItemListener() {
                        public void itemStateChanged(java.awt.event.ItemEvent evt) {
                                nounrmCbItemStateChanged(evt);
                        }
                });

                stageProgLab.setText(resourceMap.getString("stageProgLab.text")); // NOI18N
                stageProgLab.setName("stageProgLab"); // NOI18N

                addBut.setIcon(resourceMap.getIcon("addBut.icon")); // NOI18N
                addBut.setText(resourceMap.getString("addBut.text")); // NOI18N
                addBut.setName("addBut"); // NOI18N
                addBut.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                addButActionPerformed(evt);
                        }
                });

                errorLab.setText(resourceMap.getString("errorLab.text")); // NOI18N
                errorLab.setName("errorLab"); // NOI18N

                superOffCb.setText(resourceMap.getString("superOffCb.text")); // NOI18N
                superOffCb.setName("superOffCb"); // NOI18N
                superOffCb.addChangeListener(new javax.swing.event.ChangeListener() {
                        public void stateChanged(javax.swing.event.ChangeEvent evt) {
                                superOffCbStateChanged(evt);
                        }
                });
                superOffCb.addItemListener(new java.awt.event.ItemListener() {
                        public void itemStateChanged(java.awt.event.ItemEvent evt) {
                                superOffCbItemStateChanged(evt);
                        }
                });

                outputLab.setText(resourceMap.getString("outputLab.text")); // NOI18N
                outputLab.setName("outputLab"); // NOI18N

                fsProgLab.setText(resourceMap.getString("fsProgLab.text")); // NOI18N
                fsProgLab.setName("fsProgLab"); // NOI18N

                currFSLab.setText(resourceMap.getString("currFSLab.text")); // NOI18N
                currFSLab.setName("currFSLab"); // NOI18N

                superOffText.setText(resourceMap.getString("superOffText.text")); // NOI18N
                superOffText.setEnabled(false);
                superOffText.setName("superOffText"); // NOI18N

                totProgBar.setName("totProgBar"); // NOI18N

                jScrollPane3.setName("jScrollPane3"); // NOI18N

                fsList.setName("fsList"); // NOI18N
                jScrollPane3.setViewportView(fsList);

                blkSizeText.setText(resourceMap.getString("blkSizeText.text")); // NOI18N
                blkSizeText.setEnabled(false);
                blkSizeText.setName("blkSizeText"); // NOI18N

                onlyZerosCb.setText(resourceMap.getString("onlyZerosCb.text")); // NOI18N
                onlyZerosCb.setName("onlyZerosCb"); // NOI18N

                nowWipingLab.setText(resourceMap.getString("nowWipingLab.text")); // NOI18N
                nowWipingLab.setName("nowWipingLab"); // NOI18N

                exitBut.setIcon(resourceMap.getIcon("exitBut.icon")); // NOI18N
                exitBut.setText(resourceMap.getString("exitBut.text")); // NOI18N
                exitBut.setName("exitBut"); // NOI18N
                exitBut.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                exitButActionPerformed(evt);
                        }
                });

                jScrollPane2.setName("jScrollPane2"); // NOI18N

                progErrors.setColumns(20);
                progErrors.setEditable(false);
                progErrors.setRows(5);
                progErrors.setName("progErrors"); // NOI18N
                jScrollPane2.setViewportView(progErrors);

                startBut.setIcon(resourceMap.getIcon("startBut.icon")); // NOI18N
                startBut.setText(resourceMap.getString("startBut.text")); // NOI18N
                startBut.setName("startBut"); // NOI18N
                startBut.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                startButActionPerformed(evt);
                        }
                });

                ioctlCb.setText(resourceMap.getString("ioctlCb.text")); // NOI18N
                ioctlCb.setName("ioctlCb"); // NOI18N
                ioctlCb.addChangeListener(new javax.swing.event.ChangeListener() {
                        public void stateChanged(javax.swing.event.ChangeEvent evt) {
                                ioctlCbStateChanged(evt);
                        }
                });
                ioctlCb.addItemListener(new java.awt.event.ItemListener() {
                        public void itemStateChanged(java.awt.event.ItemEvent evt) {
                                ioctlCbItemStateChanged(evt);
                        }
                });

                iterCb.setText(resourceMap.getString("iterCb.text")); // NOI18N
                iterCb.setName("iterCb"); // NOI18N
                iterCb.addChangeListener(new javax.swing.event.ChangeListener() {
                        public void stateChanged(javax.swing.event.ChangeEvent evt) {
                                iterCbStateChanged(evt);
                        }
                });
                iterCb.addItemListener(new java.awt.event.ItemListener() {
                        public void itemStateChanged(java.awt.event.ItemEvent evt) {
                                iterCbItemStateChanged(evt);
                        }
                });

                forceCb.setText(resourceMap.getString("forceCb.text")); // NOI18N
                forceCb.setName("forceCb"); // NOI18N
                forceCb.addChangeListener(new javax.swing.event.ChangeListener() {
                        public void stateChanged(javax.swing.event.ChangeEvent evt) {
                                forceCbStateChanged(evt);
                        }
                });
                forceCb.addItemListener(new java.awt.event.ItemListener() {
                        public void itemStateChanged(java.awt.event.ItemEvent evt) {
                                forceCbItemStateChanged(evt);
                        }
                });

                nowfsCb.setText(resourceMap.getString("nowfsCb.text")); // NOI18N
                nowfsCb.setName("nowfsCb"); // NOI18N
                nowfsCb.addChangeListener(new javax.swing.event.ChangeListener() {
                        public void stateChanged(javax.swing.event.ChangeEvent evt) {
                                nowfsCbStateChanged(evt);
                        }
                });
                nowfsCb.addItemListener(new java.awt.event.ItemListener() {
                        public void itemStateChanged(java.awt.event.ItemEvent evt) {
                                nowfsCbItemStateChanged(evt);
                        }
                });

                iterText.setText(resourceMap.getString("iterText.text")); // NOI18N
                iterText.setEnabled(false);
                iterText.setName("iterText"); // NOI18N

                stageProgBar.setName("stageProgBar"); // NOI18N

                stopBut.setIcon(resourceMap.getIcon("stopBut.icon")); // NOI18N
                stopBut.setText(resourceMap.getString("stopBut.text")); // NOI18N
                stopBut.setEnabled(false);
                stopBut.setName("stopBut"); // NOI18N
                stopBut.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                stopButActionPerformed(evt);
                        }
                });

                loadConfBut.setIcon(resourceMap.getIcon("loadConfBut.icon")); // NOI18N
                loadConfBut.setText(resourceMap.getString("loadConfBut.text")); // NOI18N
                loadConfBut.setName("loadConfBut"); // NOI18N
                loadConfBut.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                loadConfButActionPerformed(evt);
                        }
                });

                nopartCb.setText(resourceMap.getString("nopartCb.text")); // NOI18N
                nopartCb.setName("nopartCb"); // NOI18N
                nopartCb.addChangeListener(new javax.swing.event.ChangeListener() {
                        public void stateChanged(javax.swing.event.ChangeEvent evt) {
                                nopartCbStateChanged(evt);
                        }
                });
                nopartCb.addItemListener(new java.awt.event.ItemListener() {
                        public void itemStateChanged(java.awt.event.ItemEvent evt) {
                                nopartCbItemStateChanged(evt);
                        }
                });

                fontSizeLab.setText(resourceMap.getString("fontSizeLab.text")); // NOI18N
                fontSizeLab.setName("fontSizeLab"); // NOI18N

                optLabel.setText(resourceMap.getString("optLabel.text")); // NOI18N
                optLabel.setName("optLabel"); // NOI18N

                lastZeroCb.setText(resourceMap.getString("lastZeroCb.text")); // NOI18N
                lastZeroCb.setName("lastZeroCb"); // NOI18N

                delBut.setIcon(resourceMap.getIcon("delBut.icon")); // NOI18N
                delBut.setText(resourceMap.getString("delBut.text")); // NOI18N
                delBut.setName("delBut"); // NOI18N
                delBut.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                delButActionPerformed(evt);
                        }
                });

                java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("BogDroSoft/wfs/resources/WipeFreeSpaceGUIView"); // NOI18N
                pathLabel.setText(bundle.getString("path_to_wfs")); // NOI18N
                pathLabel.setName("pathLabel"); // NOI18N

                pathField.setText(resourceMap.getString("pathField.text")); // NOI18N
                pathField.setName("pathField"); // NOI18N

                methodCb.setText(bundle.getString("wiping_method")); // NOI18N
                methodCb.setName("methodCb"); // NOI18N
                methodCb.addChangeListener(new javax.swing.event.ChangeListener() {
                        public void stateChanged(javax.swing.event.ChangeEvent evt) {
                                methodCbStateChanged(evt);
                        }
                });
                methodCb.addItemListener(new java.awt.event.ItemListener() {
                        public void itemStateChanged(java.awt.event.ItemEvent evt) {
                                methodCbItemStateChanged(evt);
                        }
                });

                methodList.setEditable(true);
                methodList.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Gutmann", "random", "DoD", "Schneier" }));
                methodList.setEnabled(false);
                methodList.setName("methodList"); // NOI18N

                wfsBrowseBut.setText(bundle.getString("browse_but")); // NOI18N
                wfsBrowseBut.setName("wfsBrowseBut"); // NOI18N
                wfsBrowseBut.addActionListener(new java.awt.event.ActionListener() {
                        public void actionPerformed(java.awt.event.ActionEvent evt) {
                                wfsBrowseButActionPerformed(evt);
                        }
                });

                javax.swing.GroupLayout contentPanelLayout = new javax.swing.GroupLayout(contentPanel);
                contentPanel.setLayout(contentPanelLayout);
                contentPanelLayout.setHorizontalGroup(
                        contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(contentPanelLayout.createSequentialGroup()
                                .addContainerGap()
                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                .addComponent(fontSizeLab)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(fontSizeSpin, javax.swing.GroupLayout.PREFERRED_SIZE, 61, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(pathLabel, javax.swing.GroupLayout.PREFERRED_SIZE, 202, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(pathField, javax.swing.GroupLayout.PREFERRED_SIZE, 288, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                .addGap(12, 12, 12)
                                                .addComponent(wfsBrowseBut))
                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                .addComponent(nowWipingLab)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(currFSLab))
                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                        .addComponent(totProgLab)
                                                        .addComponent(fsProgLab)
                                                        .addComponent(stageProgLab))
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                                        .addComponent(stageProgBar, javax.swing.GroupLayout.DEFAULT_SIZE, 780, Short.MAX_VALUE)
                                                        .addComponent(fsProgBar, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                                        .addComponent(totProgBar, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 609, Short.MAX_VALUE)
                                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                                .addGap(15, 15, 15)
                                                                .addComponent(errorLab)
                                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 246, Short.MAX_VALUE))
                                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                                .addGap(18, 18, 18)
                                                                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 356, Short.MAX_VALUE))))
                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, contentPanelLayout.createSequentialGroup()
                                                                .addComponent(optLabel)
                                                                .addGap(287, 287, 287))
                                                        .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                                .addGroup(contentPanelLayout.createSequentialGroup()
                                                                        .addComponent(methodCb)
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addComponent(methodList, javax.swing.GroupLayout.PREFERRED_SIZE, 156, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                .addComponent(ioctlCb)
                                                                .addComponent(onlyZerosCb)
                                                                .addComponent(nopartCb)
                                                                .addGroup(contentPanelLayout.createSequentialGroup()
                                                                        .addComponent(blkSizeCb)
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addComponent(blkSizeText, javax.swing.GroupLayout.PREFERRED_SIZE, 82, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                .addComponent(forceCb)
                                                                .addGroup(contentPanelLayout.createSequentialGroup()
                                                                        .addComponent(iterCb)
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addComponent(iterText, javax.swing.GroupLayout.PREFERRED_SIZE, 65, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                .addComponent(lastZeroCb)
                                                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                                                        .addGroup(javax.swing.GroupLayout.Alignment.LEADING, contentPanelLayout.createSequentialGroup()
                                                                                .addComponent(nowfsCb)
                                                                                .addGap(54, 54, 54))
                                                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                                                .addComponent(nounrmCb)
                                                                                .addGap(60, 60, 60)))
                                                                .addGroup(contentPanelLayout.createSequentialGroup()
                                                                        .addComponent(superOffCb)
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addComponent(superOffText, javax.swing.GroupLayout.PREFERRED_SIZE, 72, javax.swing.GroupLayout.PREFERRED_SIZE))))
                                                .addGap(46, 46, 46)
                                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                                        .addComponent(jScrollPane3, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 286, Short.MAX_VALUE)
                                                        .addComponent(loadConfBut, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                                        .addComponent(saveConfBut, javax.swing.GroupLayout.DEFAULT_SIZE, 286, Short.MAX_VALUE)
                                                        .addComponent(fsListLab))
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                                        .addComponent(addBut)
                                                        .addComponent(startBut)
                                                        .addComponent(stopBut)
                                                        .addComponent(aboutBut)
                                                        .addComponent(delBut)
                                                        .addComponent(exitBut))
                                                .addGap(12, 12, 12))
                                        .addComponent(outputLab))
                                .addGap(0, 0, 0))
                );

                contentPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {aboutBut, addBut, delBut, exitBut, startBut, stopBut});

                contentPanelLayout.linkSize(javax.swing.SwingConstants.HORIZONTAL, new java.awt.Component[] {loadConfBut, saveConfBut});

                contentPanelLayout.setVerticalGroup(
                        contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addGroup(contentPanelLayout.createSequentialGroup()
                                .addGap(17, 17, 17)
                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                        .addComponent(pathLabel)
                                        .addComponent(fontSizeLab)
                                        .addComponent(fontSizeSpin, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                        .addComponent(wfsBrowseBut)
                                        .addComponent(pathField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addGap(11, 11, 11)
                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                                                        .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                                                .addGroup(contentPanelLayout.createSequentialGroup()
                                                                        .addGap(17, 17, 17)
                                                                        .addComponent(onlyZerosCb)
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                                                                .addComponent(superOffCb)
                                                                                .addComponent(superOffText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                                                                .addComponent(blkSizeCb)
                                                                                .addComponent(blkSizeText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addComponent(forceCb)
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                                                                .addComponent(iterCb)
                                                                                .addComponent(iterText, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addComponent(lastZeroCb)
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addComponent(nopartCb)
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addComponent(nounrmCb)
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addComponent(nowfsCb)
                                                                        .addGap(4, 4, 4)
                                                                        .addComponent(ioctlCb)
                                                                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                        .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                                                                .addComponent(methodCb)
                                                                                .addComponent(methodList, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                                                                .addComponent(optLabel))
                                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                                .addGap(4, 4, 4)
                                                                .addComponent(fsListLab)
                                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                .addComponent(jScrollPane3, javax.swing.GroupLayout.PREFERRED_SIZE, 151, javax.swing.GroupLayout.PREFERRED_SIZE)
                                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                .addComponent(loadConfBut)
                                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                                .addComponent(saveConfBut)))
                                                .addGap(18, 18, 18)
                                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                                        .addComponent(nowWipingLab)
                                                        .addComponent(currFSLab)))
                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                .addGap(26, 26, 26)
                                                .addComponent(addBut)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(delBut)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(startBut)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(stopBut)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(aboutBut)
                                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                                .addComponent(exitBut)))
                                .addGap(21, 21, 21)
                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(stageProgLab)
                                        .addComponent(stageProgBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(fsProgLab)
                                        .addComponent(fsProgBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(totProgLab)
                                        .addComponent(totProgBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addComponent(errorLab)
                                        .addComponent(outputLab))
                                .addGroup(contentPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                .addGap(18, 18, 18)
                                                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 126, Short.MAX_VALUE))
                                        .addGroup(contentPanelLayout.createSequentialGroup()
                                                .addGap(19, 19, 19)
                                                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 125, Short.MAX_VALUE)))
                                .addContainerGap())
                );

                fontSizeSpin.getAccessibleContext().setAccessibleName(resourceMap.getString("fontSizeSpin.AccessibleContext.accessibleName")); // NOI18N
                fontSizeSpin.getAccessibleContext().setAccessibleDescription(resourceMap.getString("fontSizeSpin.AccessibleContext.accessibleDescription")); // NOI18N

                contentScrollPane.setViewportView(contentPanel);

                javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
                mainPanel.setLayout(mainPanelLayout);
                mainPanelLayout.setHorizontalGroup(
                        mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(contentScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 998, Short.MAX_VALUE)
                );
                mainPanelLayout.setVerticalGroup(
                        mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(contentScrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 629, Short.MAX_VALUE)
                );

                setComponent(mainPanel);
        }// </editor-fold>//GEN-END:initComponents

	private void exitButActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_exitButActionPerformed
	{//GEN-HEADEREND:event_exitButActionPerformed
		WipeFreeSpaceGUI.getApplication ().exit (evt);
	}//GEN-LAST:event_exitButActionPerformed

	private void superOffCbItemStateChanged (java.awt.event.ItemEvent evt)//GEN-FIRST:event_superOffCbItemStateChanged
	{//GEN-HEADEREND:event_superOffCbItemStateChanged
		if ( evt.getStateChange () == ItemEvent.SELECTED )
		{
			superOffText.setEnabled (true);
			superOffText.requestFocusInWindow ();
		}
		else superOffText.setEnabled (false);
	}//GEN-LAST:event_superOffCbItemStateChanged

	private void superOffCbStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_superOffCbStateChanged
	{//GEN-HEADEREND:event_superOffCbStateChanged
		if ( superOffCb.isSelected () )
		{
			superOffText.setEnabled (true);
			superOffText.requestFocusInWindow ();
		}
		else superOffText.setEnabled (false);
	}//GEN-LAST:event_superOffCbStateChanged

	private void blkSizeCbItemStateChanged (java.awt.event.ItemEvent evt)//GEN-FIRST:event_blkSizeCbItemStateChanged
	{//GEN-HEADEREND:event_blkSizeCbItemStateChanged
		if ( evt.getStateChange () == ItemEvent.SELECTED )
		{
			blkSizeText.setEnabled (true);
			blkSizeText.requestFocusInWindow ();
		}
		else blkSizeText.setEnabled (false);
	}//GEN-LAST:event_blkSizeCbItemStateChanged

	private void blkSizeCbStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_blkSizeCbStateChanged
	{//GEN-HEADEREND:event_blkSizeCbStateChanged
		if ( blkSizeCb.isSelected () )
		{
			blkSizeText.setEnabled (true);
			blkSizeText.requestFocusInWindow ();
		}
		else blkSizeText.setEnabled (false);
	}//GEN-LAST:event_blkSizeCbStateChanged

	private void iterCbItemStateChanged (java.awt.event.ItemEvent evt)//GEN-FIRST:event_iterCbItemStateChanged
	{//GEN-HEADEREND:event_iterCbItemStateChanged
		if ( evt.getStateChange () == ItemEvent.SELECTED )
		{
			iterText.setEnabled (true);
			iterText.requestFocusInWindow ();
		}
		else iterText.setEnabled (false);
	}//GEN-LAST:event_iterCbItemStateChanged

	private void iterCbStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_iterCbStateChanged
	{//GEN-HEADEREND:event_iterCbStateChanged
		if ( iterCb.isSelected () )
		{
			iterText.setEnabled (true);
			iterText.requestFocusInWindow ();
		}
		else iterText.setEnabled (false);
	}//GEN-LAST:event_iterCbStateChanged

	private void ioctlCbItemStateChanged (java.awt.event.ItemEvent evt)//GEN-FIRST:event_ioctlCbItemStateChanged
	{//GEN-HEADEREND:event_ioctlCbItemStateChanged
		if ( evt.getStateChange () == ItemEvent.SELECTED ) ioctlCb.setForeground (Color.RED);
		else ioctlCb.setForeground (new Color (0, 204, 0));
	}//GEN-LAST:event_ioctlCbItemStateChanged

	private void ioctlCbStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_ioctlCbStateChanged
	{//GEN-HEADEREND:event_ioctlCbStateChanged
		if ( ioctlCb.isSelected () ) ioctlCb.setForeground (Color.RED);
		else ioctlCb.setForeground (new Color (0, 204, 0));
	}//GEN-LAST:event_ioctlCbStateChanged

	private void addButActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_addButActionPerformed
	{//GEN-HEADEREND:event_addButActionPerformed
		if ( fsChooser == null )
		{
			fsChooser = new JFileChooser ();
			fsChooser.setAcceptAllFileFilterUsed (true);
			fsChooser.setMultiSelectionEnabled (true);
			fsChooser.setDialogType (JFileChooser.OPEN_DIALOG);
		}

		if ( fsChooser.showOpenDialog (mainFrame) == JFileChooser.APPROVE_OPTION )
		{
			File[] selected = fsChooser.getSelectedFiles ();
			if ( selected != null )
			{
				ListModel model = fsList.getModel ();
				Vector<String> values = new Vector<String> (selected.length*2);
				if ( model != null )
				{
					for ( int i=0; i < model.getSize (); i++ )
					{
						values.add (model.getElementAt (i).toString ());
					}
				}
				for ( int i=0; i < selected.length; i++ )
				{
					values.add (selected[i].getAbsolutePath ());
				}
				fsList.setListData (values);
			}
		}
	}//GEN-LAST:event_addButActionPerformed

	private void delButActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_delButActionPerformed
	{//GEN-HEADEREND:event_delButActionPerformed
		int[] selected = fsList.getSelectedIndices ();
		if ( selected != null )
		{
			ListModel model = fsList.getModel ();
			Vector<String> values = new Vector<String> (selected.length*2);
			if ( model != null )
			{
				boolean wasSelected;
				for ( int i=0; i < model.getSize (); i++ )
				{
					wasSelected = false;
					for ( int j=0; j < selected.length; j++ )
					{
						if ( selected[j] == i )
						{
							wasSelected = true;
							break;
						}
					}
					// only add if wasn't selected for removing
					if (! wasSelected) values.add (model.getElementAt (i).toString ());
				}
			}
			fsList.setListData (values);
		}
	}//GEN-LAST:event_delButActionPerformed

	private void startButActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_startButActionPerformed
	{//GEN-HEADEREND:event_startButActionPerformed
		if ( nopartCb.isSelected () && nounrmCb.isSelected () && nowfsCb.isSelected () )
		{
			try
			{
				JOptionPane.showMessageDialog (mainFrame, noWorkMsg, errorTitle,
					JOptionPane.ERROR_MESSAGE);
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex,
						"startButActionPerformed: " + noWorkMsg);	// NOI18N
				}
			return;
		}
		ListModel model = fsList.getModel ();
		if ( model != null )
		{
			if ( model.getSize () == 0 )
			{
				try
				{
					JOptionPane.showMessageDialog (mainFrame, noFSMsg, errorTitle,
						JOptionPane.ERROR_MESSAGE);
				}
				catch (Exception ex)
				{
					WipeFreeSpaceGUI.handleException (ex,
						"startButActionPerformed: " + noFSMsg);		// NOI18N
				}
				return;
			}
		}
		else return;
		Runtime run = Runtime.getRuntime ();
		if ( run == null )
		{
			try
			{
				JOptionPane.showMessageDialog (mainFrame, cantExecMsg, errorTitle,
					JOptionPane.ERROR_MESSAGE);
			}
			catch (Exception ex)
			{
				WipeFreeSpaceGUI.handleException (ex,
					"startButActionPerformed: " + cantExecMsg);		// NOI18N
			}
			return;
		}

		// read the checkboxes and add parameters
		Vector<String> params = new Vector<String> (20); // 15 is the minimum.
		String pathToProg = "wipefreespace";		// NOI18N
		String pathText = pathField.getText ();
		if ( pathText != null )
		{
			if ( ! pathText.isEmpty () )
			{
				pathToProg = pathText;
			}
		}
		params.add (pathToProg);
		if ( blkSizeCb.isSelected () )
		{
			try
			{
				// check if blocksize is an integer:
				Integer.parseInt (blkSizeText.getText ());
				// if we got here, it is ok
				params.add ("-B");	// NOI18N
				params.add (blkSizeText.getText ());
			}
			catch (Exception ex)
			{
				WipeFreeSpaceGUI.handleException (ex,
					"startButActionPerformed: " + blkSizeText.getText ());		// NOI18N
			}
		}
		if ( forceCb.isSelected () )
		{
			params.add ("--force");		// NOI18N
		}
		if ( ioctlCb.isSelected () )
		{
			params.add ("--use-ioctl");	// NOI18N
		}
		if ( iterCb.isSelected () )
		{
			try
			{
				// check if the number of iterations is an integer:
				Integer.parseInt (iterText.getText ());
				// if we got here, it is ok
				params.add ("-n");	// NOI18N
				params.add (iterText.getText ());
			}
			catch (Exception ex)
			{
				WipeFreeSpaceGUI.handleException (ex,
					"startButActionPerformed: " + iterText.getText ());		// NOI18N
			}
		}
		if ( lastZeroCb.isSelected () )
		{
			params.add ("--last-zero");	// NOI18N
		}
		if ( nopartCb.isSelected () )
		{
			params.add ("--nopart");	// NOI18N
		}
		if ( nounrmCb.isSelected () )
		{
			params.add ("--nounrm");	// NOI18N
		}
		if ( nowfsCb.isSelected () )
		{
			params.add ("--nowfs");		// NOI18N
		}
		if ( onlyZerosCb.isSelected () )
		{
			params.add ("--all-zeros");	// NOI18N
		}
		if ( superOffCb.isSelected () )
		{
			try
			{
				// check if the superblock offset is an integer:
				Integer.parseInt (superOffText.getText ());
				// if we got here, it is ok
				params.add ("-b");	// NOI18N
				params.add (superOffText.getText ());
			}
			catch (Exception ex)
			{
				WipeFreeSpaceGUI.handleException (ex,
					"startButActionPerformed: " + superOffText.getText ());		// NOI18N
			}
		}
		if ( methodCb.isSelected () )
		{
			params.add ("--method");	// NOI18N
			params.add (methodList.getSelectedItem ().toString ());
		}
		params.add ("--verbose");	// always use verbose output	// NOI18N
		for ( int i = 0; i < model.getSize (); i++ )
		{
			params.add (model.getElementAt (i).toString ());
		}
		// start the Process
		try
		{
			progOutput.setText (params.toString ().replaceAll (",", "") + "\n");	// NOI18N
			progErrors.setText ("");	// NOI18N
			stageProgBar.setValue (0);
			fsProgBar.setValue (0);
			totProgBar.setValue (0);
			int nStages = 3;
			if ( nopartCb.isSelected () ) nStages--;
			if ( nounrmCb.isSelected () ) nStages--;
			if ( nowfsCb.isSelected () ) nStages--;
			wfs = run.exec (params.toArray (/* just a type marker: */ new String[] {""}));	// NOI18N
			// start Threads that read the program's stdout and stderr and put
			// it in the text areas and update the progress bars accordingly.
			stdoutUpdater = new ProgressUpdater (wfs.getInputStream (), progOutput,
					stageProgBar, fsProgBar, totProgBar, currFSLab, model.getSize (),
					nStages);
			stdoutUpdater.startProcessing ();
			stderrUpdater = new ProgressUpdater (wfs.getErrorStream (), progErrors,
					null, null, null, currFSLab, model.getSize (),
					nStages);
			stderrUpdater.startProcessing ();
			// a Thread that waits for the program to finish and sets the GUI back
			/*
			wfs_monitor = new Thread (new Runnable ()
			{
				@Override
				public synchronized void run ()
				{
					try
					{
						wfs.waitFor ();
					} catch (Exception ex) {}
					WipeFreeSpaceGUI.changeGUI (new Runnable ()
					{
						@Override
						public synchronized void run ()
						{
							wfs = null;
							stopButActionPerformed (null);
						}
					});
				}
			});
			wfs_monitor.setName ("WipeFreeSpaceGUI.ProcessMonitor");	// NOI18N
			wfs_monitor.setUncaughtExceptionHandler (WipeFreeSpaceGUI.handler);
			wfs_monitor.start ();
			*/
			wfs_monitor = new SwingWorker<Void, Void> ()
			{
				@Override
				protected Void doInBackground ()
				{
					try
					{
						wfs.waitFor ();
					}
					catch (Exception ex)
					{
						WipeFreeSpaceGUI.handleException (ex,
							"startButActionPerformed: wfs_monitor: waitFor()");		// NOI18N
					}
					return null;
				}

				@Override
				protected void done ()
				{
					wfs = null;
					stopButActionPerformed (null);
				}
			};
			wfs_monitor.execute ();
		}
		catch (FileNotFoundException ex)
		{
			WipeFreeSpaceGUI.handleException (ex, "exec");	// NOI18N
			try
			{
				JOptionPane.showMessageDialog (mainFrame, wfsNotFoundMsg + ": " + ex,	// NOI18N
					errorTitle, JOptionPane.ERROR_MESSAGE);
			}
			catch (Exception exc)
			{
				WipeFreeSpaceGUI.handleException (ex,
					"startButActionPerformed: " + wfsNotFoundMsg);		// NOI18N
			}
			return;
		}
		catch (Exception ex)
		{
			WipeFreeSpaceGUI.handleException (ex, "exec");	// NOI18N
			try
			{
				JOptionPane.showMessageDialog (mainFrame, cantExecMsg + ": " + ex,	// NOI18N
					errorTitle, JOptionPane.ERROR_MESSAGE);
			}
			catch (Exception exc)
			{
				WipeFreeSpaceGUI.handleException (ex,
					"startButActionPerformed: " + cantExecMsg);		// NOI18N
			}
			return;
		}
		// change the GUI:
		stopBut.setEnabled (true);
		startBut.setEnabled (false);
		addBut.setEnabled (false);
		blkSizeCb.setEnabled (false);
		blkSizeText.setEnabled (false);
		delBut.setEnabled (false);
		forceCb.setEnabled (false);
		ioctlCb.setEnabled (false);
		iterCb.setEnabled (false);
		iterText.setEnabled (false);
		lastZeroCb.setEnabled (false);
		loadConfBut.setEnabled (false);
		nopartCb.setEnabled (false);
		nounrmCb.setEnabled (false);
		nowfsCb.setEnabled (false);
		onlyZerosCb.setEnabled (false);
		saveConfBut.setEnabled (false);
		superOffCb.setEnabled (false);
		superOffText.setEnabled (false);
	}//GEN-LAST:event_startButActionPerformed

	private void stopButActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_stopButActionPerformed
	{//GEN-HEADEREND:event_stopButActionPerformed
		if ( wfs != null )
		{
			// ask the user and kill the process.
			try
			{
				int res = JOptionPane.showConfirmDialog (mainFrame,
					stopProcessMsg, questionTitle,
					JOptionPane.YES_NO_CANCEL_OPTION);
				if ( res != JOptionPane.YES_OPTION ) return;
				// stop the Threads reading stdout and stderr
				if ( stdoutUpdater != null ) stdoutUpdater.stop ();
				if ( stderrUpdater != null ) stderrUpdater.stop ();
				// stop the process:
				wfs.destroy ();
				wfs = null;
				if ( wfs_monitor != null ) wfs_monitor.cancel (true);//.interrupt ();
			}
			catch (Exception ex)
			{
				WipeFreeSpaceGUI.handleException (ex,
					"stopButActionPerformed");	// NOI18N
			}
		}
		// Process is stopped. Set the GUI elements
		startBut.setEnabled (true);
		stopBut.setEnabled (false);
		stageProgBar.setValue (100);
		fsProgBar.setValue (100);
		totProgBar.setValue (100);
		currFSLab.setText ("-");	// NOI18N
		addBut.setEnabled (true);
		delBut.setEnabled (true);
		blkSizeCb.setEnabled (true);
		if ( blkSizeCb.isSelected() ) blkSizeText.setEnabled (true);
		if ( ! fsList.isSelectionEmpty () ) delBut.setEnabled (true);
		forceCb.setEnabled (true);
		ioctlCb.setEnabled (true);
		iterCb.setEnabled (true);
		if ( iterCb.isSelected() ) iterText.setEnabled (true);
		lastZeroCb.setEnabled (true);
		loadConfBut.setEnabled (true);
		nopartCb.setEnabled (true);
		nounrmCb.setEnabled (true);
		nowfsCb.setEnabled (true);
		onlyZerosCb.setEnabled (true);
		saveConfBut.setEnabled (true);
		superOffCb.setEnabled (true);
		if ( superOffCb.isSelected() ) superOffText.setEnabled (true);
	}//GEN-LAST:event_stopButActionPerformed

	private void forceCbItemStateChanged (java.awt.event.ItemEvent evt)//GEN-FIRST:event_forceCbItemStateChanged
	{//GEN-HEADEREND:event_forceCbItemStateChanged
		if ( evt.getStateChange () == ItemEvent.SELECTED ) forceCb.setForeground (Color.RED);
		else forceCb.setForeground (new Color (0, 204, 0));
	}//GEN-LAST:event_forceCbItemStateChanged

	private void forceCbStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_forceCbStateChanged
	{//GEN-HEADEREND:event_forceCbStateChanged
		if ( forceCb.isSelected () ) forceCb.setForeground (Color.RED);
		else forceCb.setForeground (new Color (0, 204, 0));
	}//GEN-LAST:event_forceCbStateChanged

	private void loadConfButActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_loadConfButActionPerformed
	{//GEN-HEADEREND:event_loadConfButActionPerformed
		if ( cfgFC == null )
		{
			cfgFC = new JFileChooser ();
			cfgFC.setFileFilter ( new javax.swing.filechooser.FileFilter ()
			{
				@Override
				public boolean accept ( File f )
				{
					if ( f.isDirectory () ) return true;
					if ( f.getName ().endsWith (".cfg"))	// NOI18N
					{
						return true;
					}
					return false;
				}

				@Override
				public String getDescription ()
				{
					return WFSConfFiles + " (*.cfg)";	// NOI18N
				}
			} );
		}

		cfgFC.setAcceptAllFileFilterUsed (false);
		cfgFC.setMultiSelectionEnabled (false);
		cfgFC.setDialogType (JFileChooser.OPEN_DIALOG);
		int dialogRes = cfgFC.showOpenDialog (mainFrame);

		if ( dialogRes == JFileChooser.APPROVE_OPTION )
		{
			readConfig (cfgFC.getSelectedFile ());
		}
	}//GEN-LAST:event_loadConfButActionPerformed

	private void saveConfButActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_saveConfButActionPerformed
	{//GEN-HEADEREND:event_saveConfButActionPerformed
		if ( cfgFC == null )
		{
			cfgFC = new JFileChooser ();
			cfgFC.setFileFilter ( new javax.swing.filechooser.FileFilter ()
			{
				@Override
				public boolean accept ( File f )
				{
					if ( f.isDirectory () ) return true;
					if ( f.getName ().endsWith (".cfg"))	// NOI18N
					{
						return true;
					}
					return false;
				}

				@Override
				public String getDescription ()
				{
					return WFSConfFiles + " (*.cfg)";	// NOI18N
				}
			} );
		}

		cfgFC.setAcceptAllFileFilterUsed (false);
		cfgFC.setMultiSelectionEnabled (false);
		cfgFC.setDialogType (JFileChooser.SAVE_DIALOG);
		int dialogRes = cfgFC.showSaveDialog (mainFrame);

		if ( dialogRes == JFileChooser.APPROVE_OPTION )
		{
			File newConf = cfgFC.getSelectedFile ();
			if ( newConf == null ) return;
			if ( newConf.getAbsolutePath ().length () == 0 ) return;
			try
			{
				if ( newConf.exists () )
				{
					int res = JOptionPane.showConfirmDialog (mainFrame,
						newConf.getAbsolutePath () + " " + exOverString, questionTitle,	// NOI18N
						JOptionPane.YES_NO_CANCEL_OPTION);
					if ( res != JOptionPane.YES_OPTION ) return;
				}
				if ( newConf.exists () && ! newConf.canWrite () )
				{
					JOptionPane.showMessageDialog (mainFrame,
						fileNotWriteMsg + ": " + newConf.getName (),	// NOI18N
						errorTitle, JOptionPane.ERROR_MESSAGE);
					return;
				}
			}
			catch (Exception ex) {}
			try
			{
				ConfigFile cfg = new ConfigFile (newConf);
				cfg.setAllZeros (onlyZerosCb.isSelected ());
				if ( superOffCb.isSelected () )
				{
					try
					{
						cfg.setSuperOffset (Integer.parseInt
							(superOffText.getText ()));
					}
					catch (Exception ex)
					{
						cfg.setSuperOffset (-1);
					}
				}
				else
				{
					cfg.setSuperOffset (-1);
				}
				if ( blkSizeCb.isSelected () )
				{
					try
					{
						cfg.setBlockSize (Integer.parseInt
							(blkSizeText.getText ()));
					}
					catch (Exception ex)
					{
						cfg.setBlockSize (-1);
					}
				}
				else
				{
					cfg.setBlockSize (-1);
				}
				cfg.setForce (forceCb.isSelected ());
				if ( iterCb.isSelected () )
				{
					try
					{
						cfg.setIterations (Integer.parseInt
							(iterText.getText ()));
					}
					catch (Exception ex)
					{
						cfg.setIterations (-1);
					}
				}
				else
				{
					cfg.setIterations (-1);
				}
				cfg.setLastZero (lastZeroCb.isSelected ());
				cfg.setWipePart (nopartCb.isSelected ());
				cfg.setWipeUndel (nounrmCb.isSelected ());
				cfg.setWipeFreeSpace (nowfsCb.isSelected ());
				cfg.setIoctl (ioctlCb.isSelected ());
				cfg.setX (mainFrame.getX ());
				cfg.setY (mainFrame.getY ());
				cfg.setWidth (mainFrame.getWidth ());
				cfg.setHeight (mainFrame.getHeight ());
				cfg.setIsMaximized ((mainFrame.getExtendedState () & JFrame.MAXIMIZED_BOTH) != 0);
				ListModel model = fsList.getModel ();
				Vector<String> values;
				if ( model != null )
				{
					values = new Vector<String> (model.getSize ());
					for ( int i=0; i < model.getSize (); i++ )
					{
						values.add (model.getElementAt (i).toString ());
					}
				}
				else
				{
					values = new Vector<String> (1);
				}
				cfg.setFSList (values);
				Object val = fontSizeSpin.getValue ();
				if ( val != null && val instanceof Number )
					cfg.setFontSizeValue (((Number)val).intValue ());
				cfg.setWfsPath (pathField.getText ());
				cfg.setIsMethodSelected (methodCb.isSelected ());
				cfg.setMethodName (methodList.getSelectedItem ().toString ());
				cfg.write ();
			}
			catch (Exception ex)
			{
				WipeFreeSpaceGUI.handleException (ex, "config.save");	// NOI18N
			}
		}
	}//GEN-LAST:event_saveConfButActionPerformed

	private void nopartCbItemStateChanged (java.awt.event.ItemEvent evt)//GEN-FIRST:event_nopartCbItemStateChanged
	{//GEN-HEADEREND:event_nopartCbItemStateChanged
		if ( evt.getStateChange () == ItemEvent.SELECTED )
		{
			if ( nounrmCb.isSelected () && nowfsCb.isSelected () )
				nopartCb.setSelected (false);
		}
	}//GEN-LAST:event_nopartCbItemStateChanged

	private void nopartCbStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_nopartCbStateChanged
	{//GEN-HEADEREND:event_nopartCbStateChanged
		if ( nopartCb.isSelected () )
		{
			if ( nounrmCb.isSelected () && nowfsCb.isSelected () )
				nopartCb.setSelected (false);
		}
	}//GEN-LAST:event_nopartCbStateChanged

	private void nounrmCbItemStateChanged (java.awt.event.ItemEvent evt)//GEN-FIRST:event_nounrmCbItemStateChanged
	{//GEN-HEADEREND:event_nounrmCbItemStateChanged
		if ( evt.getStateChange () == ItemEvent.SELECTED )
		{
			if ( nopartCb.isSelected () && nowfsCb.isSelected () )
				nounrmCb.setSelected (false);
		}
	}//GEN-LAST:event_nounrmCbItemStateChanged

	private void nounrmCbStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_nounrmCbStateChanged
	{//GEN-HEADEREND:event_nounrmCbStateChanged
		if ( nounrmCb.isSelected () )
		{
			if ( nopartCb.isSelected () && nowfsCb.isSelected () )
				nounrmCb.setSelected (false);
		}
	}//GEN-LAST:event_nounrmCbStateChanged

	private void nowfsCbItemStateChanged (java.awt.event.ItemEvent evt)//GEN-FIRST:event_nowfsCbItemStateChanged
	{//GEN-HEADEREND:event_nowfsCbItemStateChanged
		if ( evt.getStateChange () == ItemEvent.SELECTED )
		{
			if ( nopartCb.isSelected () && nounrmCb.isSelected () )
				nowfsCb.setSelected (false);
		}
	}//GEN-LAST:event_nowfsCbItemStateChanged

	private void nowfsCbStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_nowfsCbStateChanged
	{//GEN-HEADEREND:event_nowfsCbStateChanged
		if ( nowfsCb.isSelected () )
		{
			if ( nopartCb.isSelected () && nounrmCb.isSelected () )
				nowfsCb.setSelected (false);
		}
	}//GEN-LAST:event_nowfsCbStateChanged

	private void fontSizeSpinStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_fontSizeSpinStateChanged
	{//GEN-HEADEREND:event_fontSizeSpinStateChanged
		Object val = fontSizeSpin.getValue ();
		if ( val != null && val instanceof Number )
			WipeFreeSpaceGUI.setFontSize (mainFrame, ((Number)val).floatValue ());
	}//GEN-LAST:event_fontSizeSpinStateChanged

	private void methodCbStateChanged (javax.swing.event.ChangeEvent evt)//GEN-FIRST:event_methodCbStateChanged
	{//GEN-HEADEREND:event_methodCbStateChanged
		if ( methodCb.isSelected () )
		{
			methodList.setEnabled (true);
		}
		else
		{
			methodList.setEnabled (false);
		}
	}//GEN-LAST:event_methodCbStateChanged

	private void methodCbItemStateChanged (java.awt.event.ItemEvent evt)//GEN-FIRST:event_methodCbItemStateChanged
	{//GEN-HEADEREND:event_methodCbItemStateChanged
		if ( evt.getStateChange () == ItemEvent.SELECTED )
		{
			methodList.setEnabled (true);
		}
		else if ( evt.getStateChange () == ItemEvent.DESELECTED )
		{
			methodList.setEnabled (false);
		}
	}//GEN-LAST:event_methodCbItemStateChanged

	private void wfsBrowseButActionPerformed (java.awt.event.ActionEvent evt)//GEN-FIRST:event_wfsBrowseButActionPerformed
	{//GEN-HEADEREND:event_wfsBrowseButActionPerformed
		if ( progFC == null )
		{
			progFC = new JFileChooser ();
			progFC.setFileFilter ( new javax.swing.filechooser.FileFilter ()
			{
				@Override
				public boolean accept ( File f )
				{
					if ( f.isDirectory () ) return true;
					if ( f.getName ().toLowerCase (Locale.ENGLISH).contains ("wipefreespace"))	// NOI18N
					{
						return true;
					}
					return false;
				}

				@Override
				public String getDescription ()
				{
					return "WipeFreeSpace";	// NOI18N
				}
			} );
		}

		progFC.setAcceptAllFileFilterUsed (true);
		progFC.setMultiSelectionEnabled (false);
		progFC.setDialogType (JFileChooser.OPEN_DIALOG);
		int dialogRes = progFC.showOpenDialog (mainFrame);

		if ( dialogRes == JFileChooser.APPROVE_OPTION )
		{
			pathField.setText (progFC.getSelectedFile ().getAbsolutePath ());
		}
	}//GEN-LAST:event_wfsBrowseButActionPerformed

	private void readConfig (File f)
	{
		if ( f == null ) return;
		if ( (! f.exists ()) || (! f.canRead ()) ) return;

		try
		{
			ConfigFile cfg = new ConfigFile (f);
			cfg.read ();
			if ( cfg.getIsMax () ) mainFrame.setExtendedState (JFrame.MAXIMIZED_BOTH);
			else
			{
				mainFrame.setExtendedState (mainFrame.getExtendedState () & ~ JFrame.MAXIMIZED_BOTH);
				mainFrame.setLocation (cfg.getX (), cfg.getY ());
				mainFrame.setSize (cfg.getWidth (), cfg.getHeight ());
			}
			if ( cfg.getAllZeros () ) onlyZerosCb.setSelected (true);
			else onlyZerosCb.setSelected (false);
			if ( cfg.getSuperOffset () >= 0 )
			{
				superOffCb.setSelected (true);
				superOffText.setText (String.valueOf (cfg.getSuperOffset ()));
			}
			else superOffCb.setSelected (false);
			if ( cfg.getBlockSize () >= 0 )
			{
				blkSizeCb.setSelected (true);
				blkSizeText.setText (String.valueOf (cfg.getBlockSize ()));
			}
			else blkSizeCb.setSelected (false);
			if ( cfg.getForce () ) forceCb.setSelected (true);
			else forceCb.setSelected (false);
			if ( cfg.getIterations () >= 0 )
			{
				iterCb.setSelected (true);
				iterText.setText (String.valueOf (cfg.getIterations ()));
			}
			else iterCb.setSelected (false);
			if ( cfg.getLastZero () ) lastZeroCb.setSelected (true);
			else lastZeroCb.setSelected (false);
			if ( cfg.getWipePart () ) nopartCb.setSelected (true);
			else nopartCb.setSelected (false);
			if ( cfg.getWipeUndel () ) nounrmCb.setSelected (true);
			else nounrmCb.setSelected (false);
			if ( cfg.getWipeFreeSpace () ) nowfsCb.setSelected (true);
			else nowfsCb.setSelected (false);
			if ( cfg.getIoctl () ) ioctlCb.setSelected (true);
			else ioctlCb.setSelected (false);
			Vector<String> fslist = cfg.getFSList ();
			if ( fslist != null )
			{
				if ( fslist.size () > 0 ) fsList.setListData (fslist);
			}
			fontSizeSpin.setValue ((float)cfg.getFontSizeValue ());
			if ( cfg.getIsMethod () ) methodCb.setSelected (true);
			else methodCb.setSelected (false);
			pathField.setText (cfg.getWfsPath ());
			methodList.setSelectedItem (cfg.getMethodName ());
		}
		catch (Exception ex)
		{
			WipeFreeSpaceGUI.handleException (ex, "readConfig: " + f.getAbsolutePath ());	// NOI18N
		}

		if ( (mainFrame.getExtendedState () & JFrame.MAXIMIZED_BOTH) == 0 )
		{
			// if not maximized, verify position and size
			GraphicsConfiguration gc = null;
			Insets is = null;
			try
			{
				gc = mainFrame.getGraphicsConfiguration ();
				Toolkit tk = Toolkit.getDefaultToolkit ();
				if ( tk != null ) is = tk.getScreenInsets (gc);
			}
			catch (Exception ex)
			{
				WipeFreeSpaceGUI.handleException (ex,
					"readConfig: GraphicsConfiguration/Toolkit");	// NOI18N
			}
			int maxX = 800;
			int maxY = 600;
			if ( gc != null )
			{
				Rectangle bounds = gc.getBounds ();
				if ( bounds != null )
				{
					if ( is != null )
					{
						maxX = bounds.width - is.left - is.right;
						maxY = bounds.height - is.top - is.bottom;
					}
					else
					{
						maxX = bounds.width;
						maxY = bounds.height;
					}
				}
			}
			if ( mainFrame.getWidth () <= 0 )
			{
				mainFrame.setSize (maxX, mainFrame.getHeight ());
			}
			if ( mainFrame.getHeight () <= 0 )
			{
				mainFrame.setSize (mainFrame.getWidth (), maxY);
			}
			if ( mainFrame.getX () + mainFrame.getWidth () < 0
				|| mainFrame.getX () > maxX )
			{
				mainFrame.setLocation (0, mainFrame.getY ());
			}
			if ( mainFrame.getY () + mainFrame.getHeight () < 0
				|| mainFrame.getY () > maxY )
			{
				mainFrame.setLocation (mainFrame.getX (), 0);
			}
		}
	}

        // Variables declaration - do not modify//GEN-BEGIN:variables
        private javax.swing.JButton aboutBut;
        private javax.swing.JButton addBut;
        private javax.swing.JCheckBox blkSizeCb;
        private javax.swing.JTextField blkSizeText;
        private javax.swing.JPanel contentPanel;
        private javax.swing.JScrollPane contentScrollPane;
        private javax.swing.JLabel currFSLab;
        private javax.swing.JButton delBut;
        private javax.swing.JLabel errorLab;
        private javax.swing.JButton exitBut;
        private javax.swing.JLabel fontSizeLab;
        private javax.swing.JSpinner fontSizeSpin;
        private javax.swing.JCheckBox forceCb;
        private javax.swing.JList fsList;
        private javax.swing.JLabel fsListLab;
        private javax.swing.JProgressBar fsProgBar;
        private javax.swing.JLabel fsProgLab;
        private javax.swing.JCheckBox ioctlCb;
        private javax.swing.JCheckBox iterCb;
        private javax.swing.JTextField iterText;
        private javax.swing.JScrollPane jScrollPane1;
        private javax.swing.JScrollPane jScrollPane2;
        private javax.swing.JScrollPane jScrollPane3;
        private javax.swing.JCheckBox lastZeroCb;
        private javax.swing.JButton loadConfBut;
        private javax.swing.JPanel mainPanel;
        private javax.swing.JCheckBox methodCb;
        private javax.swing.JComboBox methodList;
        private javax.swing.JCheckBox nopartCb;
        private javax.swing.JCheckBox nounrmCb;
        private javax.swing.JLabel nowWipingLab;
        private javax.swing.JCheckBox nowfsCb;
        private javax.swing.JCheckBox onlyZerosCb;
        private javax.swing.JLabel optLabel;
        private javax.swing.JLabel outputLab;
        private javax.swing.JTextField pathField;
        private javax.swing.JLabel pathLabel;
        private javax.swing.JTextArea progErrors;
        private javax.swing.JTextArea progOutput;
        private javax.swing.JButton saveConfBut;
        private javax.swing.JProgressBar stageProgBar;
        private javax.swing.JLabel stageProgLab;
        private javax.swing.JButton startBut;
        private javax.swing.JButton stopBut;
        private javax.swing.JCheckBox superOffCb;
        private javax.swing.JTextField superOffText;
        private javax.swing.JProgressBar totProgBar;
        private javax.swing.JLabel totProgLab;
        private javax.swing.JButton wfsBrowseBut;
        // End of variables declaration//GEN-END:variables
}
